@@ -2,6 +2,9 @@
 //
 // Copyright Bob Friesenhahn, 1999, 2002
 //
+// Copyright @ 2013 ImageMagick Studio LLC, a non-profit organization
+// dedicated to making software imaging solutions freely available.
+//
 // Implementation of STL classes and functions
 //
 
@@ -39,13 +42,16 @@ void Magick::adaptiveThresholdImage::operator()( Magick::Image &image_ ) const
 }
 
 // Add noise to image with specified noise type
-Magick::addNoiseImage::addNoiseImage( Magick::NoiseType noiseType_ )
-  : _noiseType( noiseType_ )
+Magick::addNoiseImage::addNoiseImage(const Magick::NoiseType noiseType_,
+  const double attenuate_)
+  : _noiseType(noiseType_),
+    _attenuate(attenuate_)
 {
 }
-void Magick::addNoiseImage::operator()( Magick::Image &image_ ) const
+
+void Magick::addNoiseImage::operator()(Magick::Image &image_) const
 {
-  image_.addNoise( _noiseType );
+  image_.addNoise(_noiseType,_attenuate);
 }
 
 // Transform image by specified affine (or free transform) matrix.
@@ -170,28 +176,28 @@ void Magick::cdlImage::operator()( Image &image_ ) const
   image_.cdl( _cdl.c_str() );
 }
 
-// Colorize image using pen color at specified percent opacity
-Magick::colorizeImage::colorizeImage( const unsigned int opacityRed_,
-                                      const unsigned int opacityGreen_,
-                                      const unsigned int opacityBlue_,
+// Colorize image using pen color at specified percent alpha
+Magick::colorizeImage::colorizeImage( const unsigned int alphaRed_,
+                                      const unsigned int alphaGreen_,
+                                      const unsigned int alphaBlue_,
                                       const Magick::Color &penColor_ )
-  : _opacityRed ( opacityRed_ ),
-    _opacityGreen ( opacityGreen_ ),
-    _opacityBlue ( opacityBlue_ ),
+  : _alphaRed ( alphaRed_ ),
+    _alphaGreen ( alphaGreen_ ),
+    _alphaBlue ( alphaBlue_ ),
     _penColor( penColor_ )
 {
 }
-Magick::colorizeImage::colorizeImage( const unsigned int opacity_,
+Magick::colorizeImage::colorizeImage( const unsigned int alpha_,
                                       const Magick::Color &penColor_ )
-  : _opacityRed ( opacity_ ),
-    _opacityGreen ( opacity_ ),
-    _opacityBlue ( opacity_ ),
+  : _alphaRed ( alpha_ ),
+    _alphaGreen ( alpha_ ),
+    _alphaBlue ( alpha_ ),
     _penColor( penColor_ )
 {
 }
 void Magick::colorizeImage::operator()( Magick::Image &image_ ) const
 {
-  image_.colorize( _opacityRed, _opacityGreen, _opacityBlue, _penColor );
+  image_.colorize( _alphaRed, _alphaGreen, _alphaBlue, _penColor );
 }
 
 // Apply a color matrix to the image channels.  The user supplied
@@ -294,9 +300,9 @@ void Magick::despeckleImage::operator()( Magick::Image &image_ ) const
 
 // Distort image.  distorts an image using various distortion methods, by
 // mapping color lookups of the source image to a new destination image
-// usally of the same size as the source image, unless 'bestfit' is set to
+// usually of the same size as the source image, unless 'bestfit' is set to
 // true.
-Magick::distortImage::distortImage( const Magick::DistortImageMethod method_,
+Magick::distortImage::distortImage( const Magick::DistortMethod method_,
                                     const size_t number_arguments_,
                                     const double *arguments_,
                                     const bool bestfit_ )
@@ -306,7 +312,7 @@ Magick::distortImage::distortImage( const Magick::DistortImageMethod method_,
     _bestfit( bestfit_ )
 {
 }
-Magick::distortImage::distortImage( const Magick::DistortImageMethod method_,
+Magick::distortImage::distortImage( const Magick::DistortMethod method_,
                                     const size_t number_arguments_,
                                     const double *arguments_ )
   : _method ( method_ ),
@@ -326,7 +332,7 @@ Magick::drawImage::drawImage( const Magick::Drawable &drawable_ )
 {
   _drawableList.push_back( drawable_ );
 }
-Magick::drawImage::drawImage( const std::list<Magick::Drawable> &drawable_ )
+Magick::drawImage::drawImage( const std::vector<Magick::Drawable> &drawable_ )
   : _drawableList( drawable_ )
 {
 }
@@ -335,7 +341,7 @@ void Magick::drawImage::operator()( Magick::Image &image_ ) const
   image_.draw( _drawableList );
 }
 
-// Edge image (hilight edges in image)
+// Edge image (highlight edges in image)
 Magick::edgeImage::edgeImage( const double radius_ )
   : _radius( radius_ )
 {
@@ -345,7 +351,7 @@ void Magick::edgeImage::operator()( Magick::Image &image_ ) const
   image_.edge( _radius );
 }
 
-// Emboss image (hilight edges with 3D effect)
+// Emboss image (highlight edges with 3D effect)
 Magick::embossImage::embossImage( void )
   : _radius( 1 ),
     _sigma( 0.5 )
@@ -398,115 +404,120 @@ void Magick::flipImage::operator()( Magick::Image &image_ ) const
   image_.flip( );
 }
 
-// Flood-fill image with color
-// Flood-fill color across pixels starting at target-pixel and
-// stopping at pixels matching specified border color.  Uses current
-// fuzz setting when determining color match.
-Magick::floodFillColorImage::floodFillColorImage( const ssize_t x_,
-                                                  const ssize_t y_,
-                                                  const Magick::Color &fillColor_ )
-  : _x(x_),
+Magick::floodFillAlphaImage::floodFillAlphaImage(const ssize_t x_,
+  const ssize_t y_,const unsigned int alpha_,const Color &target_,
+  const bool invert_)
+  : _target(target_),
+    _alpha(alpha_),
+    _x(x_),
     _y(y_),
-    _fillColor(fillColor_),
-    _borderColor()
+    _invert(invert_)
 {
 }
-Magick::floodFillColorImage::floodFillColorImage( const Magick::Geometry &point_,
-                                                  const Magick::Color &fillColor_ )
-  : _x(point_.xOff()),
-    _y(point_.yOff()),
-    _fillColor(fillColor_),
-    _borderColor()
-{
-}
-// Flood-fill color across pixels starting at target-pixel and
-// stopping at pixels matching specified border color.  Uses current
-// fuzz setting when determining color match.
-Magick::floodFillColorImage::floodFillColorImage( const ssize_t x_,
-                                                  const ssize_t y_,
-                                                  const Magick::Color &fillColor_,
-                                                  const Magick::Color &borderColor_ )
-  : _x(x_),
-    _y(y_),
-    _fillColor(fillColor_),
-    _borderColor(borderColor_)
-{
-}
-Magick::floodFillColorImage::floodFillColorImage( const Geometry &point_,
-                                                  const Color &fillColor_,
-                                                  const Color &borderColor_ )
-  : _x(point_.xOff()),
-    _y(point_.yOff()),
-    _fillColor(fillColor_),
-    _borderColor(borderColor_)
-{
-}
-void Magick::floodFillColorImage::operator()( Magick::Image &image_ ) const
-{
-  if ( _borderColor.isValid() )
-    {
-      image_.floodFillColor( _x, _y, _fillColor, _borderColor );
-    }
-  else
-    {
-      image_.floodFillColor( _x, _y, _fillColor );
-    }
-}
 
-// Flood-fill image with texture
+void Magick::floodFillAlphaImage::operator()(Magick::Image &image_) const
+{
+  image_.floodFillAlpha(_x,_y,_alpha,_target,_invert);
+}
 
-// Flood-fill texture across pixels that match the color of the target
-// pixel and are neighbors of the target pixel.  Uses current fuzz
-// setting when determining color match.
-Magick::floodFillTextureImage::floodFillTextureImage( const ssize_t x_,
-                                                      const ssize_t y_,
-                                                      const Magick::Image &texture_ )
+Magick::floodFillColorImage::floodFillColorImage(const ssize_t x_,
+  const ssize_t y_,const Magick::Color &fillColor_,const bool invert_)
   : _x(x_),
     _y(y_),
-    _texture(texture_),
-    _borderColor()
+    _fillColor(fillColor_),
+    _borderColor(),
+    _invert(invert_)
 {
 }
-Magick::floodFillTextureImage::floodFillTextureImage( const Magick::Geometry &point_,
-                                                      const Magick::Image &texture_ )
+
+Magick::floodFillColorImage::floodFillColorImage(
+  const Magick::Geometry &point_,const Magick::Color &fillColor_,
+  const bool invert_)
   : _x(point_.xOff()),
     _y(point_.yOff()),
-    _texture(texture_),
-    _borderColor()
+    _fillColor(fillColor_),
+    _borderColor(),
+    _invert(invert_)
 {
 }
-// Flood-fill texture across pixels starting at target-pixel and
-// stopping at pixels matching specified border color.  Uses current
-// fuzz setting when determining color match.
-Magick::floodFillTextureImage::floodFillTextureImage( const ssize_t x_,
-                                                      const ssize_t y_,
-                                                      const Magick::Image &texture_,
-                                                      const Magick::Color &borderColor_ )
+
+Magick::floodFillColorImage::floodFillColorImage(const ssize_t x_,
+  const ssize_t y_,const Magick::Color &fillColor_,
+  const Magick::Color &borderColor_,const bool invert_)
   : _x(x_),
     _y(y_),
-    _texture(texture_),
-    _borderColor(borderColor_)
+    _fillColor(fillColor_),
+    _borderColor(borderColor_),
+    _invert(invert_)
 {
 }
-Magick::floodFillTextureImage::floodFillTextureImage( const Magick::Geometry &point_,
-                                                      const Magick::Image &texture_,
-                                                      const Magick::Color &borderColor_ )
+
+Magick::floodFillColorImage::floodFillColorImage(const Geometry &point_,
+  const Color &fillColor_,const Color &borderColor_,const bool invert_)
   : _x(point_.xOff()),
     _y(point_.yOff()),
-    _texture(texture_),
-    _borderColor(borderColor_)
+    _fillColor(fillColor_),
+    _borderColor(borderColor_),
+    _invert(invert_)
 {
 }
-void Magick::floodFillTextureImage::operator()( Magick::Image &image_ ) const
+void Magick::floodFillColorImage::operator()(Magick::Image &image_) const
 {
-  if ( _borderColor.isValid() )
-    {
-      image_.floodFillTexture( _x, _y, _texture, _borderColor );
-    }
+  if (_borderColor.isValid())
+    image_.floodFillColor(_x,_y,_fillColor,_borderColor,_invert);
   else
-    {
-      image_.floodFillTexture( _x, _y, _texture );
-    }
+    image_.floodFillColor(_x,_y,_fillColor,_invert);
+}
+
+Magick::floodFillTextureImage::floodFillTextureImage(const ssize_t x_,
+  const ssize_t y_,const Magick::Image &texture_,const bool invert_)
+  : _x(x_),
+    _y(y_),
+    _texture(texture_),
+    _borderColor(),
+    _invert(invert_)
+{
+}
+
+Magick::floodFillTextureImage::floodFillTextureImage(
+  const Magick::Geometry &point_,const Magick::Image &texture_,
+  const bool invert_)
+  : _x(point_.xOff()),
+    _y(point_.yOff()),
+    _texture(texture_),
+    _borderColor(),
+    _invert(invert_)
+{
+}
+
+Magick::floodFillTextureImage::floodFillTextureImage(const ssize_t x_,
+  const ssize_t y_,const Magick::Image &texture_,
+  const Magick::Color &borderColor_,const bool invert_)
+  : _x(x_),
+    _y(y_),
+    _texture(texture_),
+    _borderColor(borderColor_),
+    _invert(invert_)
+{
+}
+
+Magick::floodFillTextureImage::floodFillTextureImage(
+  const Magick::Geometry &point_,const Magick::Image &texture_,
+  const Magick::Color &borderColor_,const bool invert_)
+  : _x(point_.xOff()),
+    _y(point_.yOff()),
+    _texture(texture_),
+    _borderColor(borderColor_),
+    _invert(invert_)
+{
+}
+
+void Magick::floodFillTextureImage::operator()(Magick::Image &image_) const
+{
+  if (_borderColor.isValid())
+    image_.floodFillTexture(_x,_y,_texture,_borderColor,_invert);
+  else
+    image_.floodFillTexture(_x,_y,_texture,_invert);
 }
 
 // Flop image (reflect each scanline in the horizontal direction)
@@ -640,22 +651,6 @@ void Magick::levelImage::operator()( Magick::Image &image_ ) const
   image_.level( _black_point, _white_point, _mid_point );
 }
 
-// Level image channel
-Magick::levelChannelImage::levelChannelImage( const Magick::ChannelType channel,                                              const double black_point,
-                                              const double white_point,
-                                              const double mid_point )
-  : _channel(channel),
-    _black_point(black_point),
-    _white_point(white_point),
-    _mid_point(mid_point)
-{
-}
-
-void Magick::levelChannelImage::operator()( Magick::Image &image_ ) const
-{
-  image_.levelChannel( _channel, _black_point, _white_point, _mid_point );
-}
-
 // Magnify image by integral size
 Magick::magnifyImage::magnifyImage( void )
 {
@@ -677,45 +672,17 @@ void Magick::mapImage::operator()( Magick::Image &image_ ) const
   image_.map( _mapImage, _dither );
 }
 
-// Floodfill designated area with a matte value
-Magick::matteFloodfillImage::matteFloodfillImage( const Color &target_ ,
-                                                  const unsigned int matte_,
-                                                  const ssize_t x_, const ssize_t y_,
-                                                  const PaintMethod method_ )
-  : _target( target_ ),
-    _matte( matte_ ),
-    _x( x_ ),
-    _y( y_ ),
-    _method( method_ )
-{
-}
-void Magick::matteFloodfillImage::operator()( Magick::Image &image_ ) const
-{
-  image_.matteFloodfill( _target, _matte, _x, _y, _method );
-}
-
 // Filter image by replacing each pixel component with the median
 // color in a circular neighborhood
-Magick::medianFilterImage::medianFilterImage( const double radius_  )
+Magick::medianConvolveImage::medianConvolveImage( const double radius_  )
   : _radius( radius_ )
 {
 }
-void Magick::medianFilterImage::operator()( Magick::Image &image_ ) const
+void Magick::medianConvolveImage::operator()( Magick::Image &image_ ) const
 {
   image_.medianFilter( _radius );
 }
 
-// Merge image layers
-Magick::mergeLayersImage::mergeLayersImage( 
-  Magick::ImageLayerMethod layerMethod_ )
-  : _layerMethod( layerMethod_ )
-{
-}
-void Magick::mergeLayersImage::operator()( Magick::Image &image_ ) const
-{
-  image_.mergeLayers( _layerMethod );
-}
-
 // Reduce image by integral size
 Magick::minifyImage::minifyImage( void )
 {
@@ -770,19 +737,19 @@ void Magick::oilPaintImage::operator()( Magick::Image &image_ ) const
   image_.oilPaint( _radius );
 }
 
-// Set or attenuate the image opacity channel. If the image pixels are
-// opaque then they are set to the specified opacity value, otherwise
-// they are blended with the supplied opacity value.  The value of
-// opacity_ ranges from 0 (completely opaque) to QuantumRange. The defines
-// OpaqueOpacity and TransparentOpacity are available to specify
+// Set or attenuate the image alpha channel. If the image pixels are
+// opaque then they are set to the specified alpha value, otherwise
+// they are blended with the supplied alpha value.  The value of
+// alpha_ ranges from 0 (completely opaque) to QuantumRange. The defines
+// OpaqueAlpha and TransparentAlpha are available to specify
 // completely opaque or completely transparent, respectively.
-Magick::opacityImage::opacityImage( const unsigned int opacity_ )
-  : _opacity( opacity_ )
+Magick::alphaImage::alphaImage( const unsigned int alpha_ )
+  : _alpha( alpha_ )
 {
 }
-void Magick::opacityImage::operator()( Magick::Image &image_ ) const
+void Magick::alphaImage::operator()( Magick::Image &image_ ) const
 {
-  image_.opacity( _opacity );
+  image_.alpha( _alpha );
 }
 
 // Change color of opaque pixel to specified pen color.
@@ -822,13 +789,15 @@ void Magick::raiseImage::operator()( Magick::Image &image_ ) const
 
 Magick::ReadOptions::ReadOptions(void)
   : _imageInfo(static_cast<ImageInfo*>(AcquireMagickMemory(
-      sizeof(ImageInfo))))
+      sizeof(ImageInfo)))),
+    _quiet(false)
 {
   GetImageInfo(_imageInfo);
 }
 
 Magick::ReadOptions::ReadOptions(const Magick::ReadOptions& options_)
-  : _imageInfo(CloneImageInfo(options_._imageInfo))
+  : _imageInfo(CloneImageInfo(options_._imageInfo)),
+    _quiet(false)
 {
 }
 
@@ -863,6 +832,26 @@ size_t Magick::ReadOptions::depth(void) const
   return(_imageInfo->depth);
 }
 
+void Magick::ReadOptions::ping(const bool flag_)
+{
+  _imageInfo->ping=(MagickBooleanType) flag_;
+}
+
+bool Magick::ReadOptions::ping(void) const
+{
+   return(static_cast<bool>(_imageInfo->ping));
+}
+
+void Magick::ReadOptions::quiet(const bool quiet_)
+{
+  _quiet=quiet_;
+}
+
+bool Magick::ReadOptions::quiet(void) const
+{
+   return(_quiet);
+}
+
 void Magick::ReadOptions::size(const Geometry &geometry_)
 {
   _imageInfo->size=(char *) RelinquishMagickMemory(_imageInfo->size);
@@ -899,10 +888,10 @@ void Magick::reduceNoiseImage::operator()( Image &image_ ) const
 }
 
 // Roll image (rolls image vertically and horizontally) by specified
-// number of columnms and rows)
+// number of columns and rows)
 Magick::rollImage::rollImage( const Magick::Geometry &roll_ )
-  : _columns( roll_.width() ),
-    _rows( roll_.height() )
+  : _columns( static_cast<ssize_t>(roll_.width()) ),
+    _rows( static_cast<ssize_t>(roll_.height()) )
 {
 }
 Magick::rollImage::rollImage( const ssize_t columns_,
@@ -1034,7 +1023,7 @@ void Magick::solarizeImage::operator()( Magick::Image &image_ ) const
   image_.solarize( _factor );
 }
 
-// Spread pixels randomly within image by specified ammount
+// Spread pixels randomly within image by specified amount
 Magick::spreadImage::spreadImage( const size_t amount_ )
   : _amount( amount_ )
 {
@@ -1105,26 +1094,6 @@ void Magick::thresholdImage::operator()( Magick::Image &image_ ) const
   image_.threshold( _threshold );
 }
 
-// Transform image based on image and crop geometries
-Magick::transformImage::transformImage( const Magick::Geometry &imageGeometry_ )
-  : _imageGeometry( imageGeometry_ ),
-    _cropGeometry( )
-{
-}
-Magick::transformImage::transformImage( const Magick::Geometry &imageGeometry_,
-                                        const Geometry &cropGeometry_  )
-  : _imageGeometry( imageGeometry_ ),
-    _cropGeometry( cropGeometry_ )
-{
-}
-void Magick::transformImage::operator()( Magick::Image &image_ ) const
-{
-  if ( _cropGeometry.isValid() )
-    image_.transform( _imageGeometry, _cropGeometry );
-  else
-    image_.transform( _imageGeometry );
-}
-
 // Set image color to transparent
 Magick::transparentImage::transparentImage( const Magick::Color& color_ )
   : _color( color_ )
@@ -1180,16 +1149,6 @@ void Magick::zoomImage::operator()( Magick::Image &image_ ) const
 // Function object image attribute accessors
 //
 
-// Anti-alias Postscript and TrueType fonts (default true)
-Magick::antiAliasImage::antiAliasImage( const bool flag_ )
-  : _flag( flag_ )
-{
-}
-void Magick::antiAliasImage::operator()( Magick::Image &image_ ) const
-{
-  image_.antiAlias( _flag );
-}
-
 // Join images into a single multi-image file
 Magick::adjoinImage::adjoinImage( const bool flag_ )
   : _flag( flag_ )
@@ -1261,52 +1220,56 @@ void Magick::boxColorImage::operator()( Magick::Image &image_ ) const
   image_.boxColor( _boxColor );
 }
 
-// Chromaticity blue primary point (e.g. x=0.15, y=0.06)
-Magick::chromaBluePrimaryImage::chromaBluePrimaryImage( const double x_,
-                                                        const double y_ )
-  : _x( x_ ),
-    _y( y_ )
+Magick::chromaBluePrimaryImage::chromaBluePrimaryImage(const double x_,
+  const double y_,const double z_)
+  : _x(x_),
+    _y(y_),
+    _z(z_)
 {
 }
-void Magick::chromaBluePrimaryImage::operator()( Magick::Image &image_ ) const
-{
-  image_.chromaBluePrimary( _x, _y );
-}
 
-// Chromaticity green primary point (e.g. x=0.3, y=0.6)
-Magick::chromaGreenPrimaryImage::chromaGreenPrimaryImage( const double x_,
-                                                          const double y_ )
-  : _x( x_ ),
-    _y( y_ )
+void Magick::chromaBluePrimaryImage::operator()(Magick::Image &image_) const
+{
+  image_.chromaBluePrimary(_x,_y,_z);
+}
+
+Magick::chromaGreenPrimaryImage::chromaGreenPrimaryImage(const double x_,
+  const double y_,const double z_)
+  : _x(x_),
+    _y(y_),
+    _z(z_)
 {
 }
+
 void Magick::chromaGreenPrimaryImage::operator()( Magick::Image &image_ ) const
 {
-  image_.chromaGreenPrimary( _x, _y );
+  image_.chromaGreenPrimary(_x,_y,_z);
 }
 
-// Chromaticity red primary point (e.g. x=0.64, y=0.33)
-Magick::chromaRedPrimaryImage::chromaRedPrimaryImage( const double x_,
-                                                      const double y_ )
-  : _x( x_ ),
-    _y( y_ )
+Magick::chromaRedPrimaryImage::chromaRedPrimaryImage(const double x_,
+  const double y_,const double z_)
+  : _x(x_),
+    _y(y_),
+    _z(z_)
 {
 }
-void Magick::chromaRedPrimaryImage::operator()( Magick::Image &image_ ) const
-{
-  image_.chromaRedPrimary( _x, _y );
-}
 
-// Chromaticity white point (e.g. x=0.3127, y=0.329)
-Magick::chromaWhitePointImage::chromaWhitePointImage( const double x_,
-                                                      const double y_ )
-  : _x( x_ ),
-    _y( y_ )
+void Magick::chromaRedPrimaryImage::operator()(Magick::Image &image_) const
+{
+  image_.chromaRedPrimary(_x,_y,_z);
+}
+
+Magick::chromaWhitePointImage::chromaWhitePointImage(const double x_,
+  const double y_,const double z_)
+  : _x(x_),
+    _y(y_),
+    _z(z_)
 {
 }
-void Magick::chromaWhitePointImage::operator()( Magick::Image &image_ ) const
+
+void Magick::chromaWhitePointImage::operator()(Magick::Image &image_) const
 {
-  image_.chromaWhitePoint( _x, _y );
+  image_.chromaWhitePoint(_x,_y,_z);
 }
 
 // Colors within this distance are considered equal
@@ -1353,13 +1316,13 @@ void Magick::compressTypeImage::operator()( Magick::Image &image_ ) const
 }
 
 // Vertical and horizontal resolution in pixels of the image
-Magick::densityImage::densityImage( const Geometry &geomery_ )
-  : _geomery( geomery_ )
+Magick::densityImage::densityImage( const Point &point_ )
+  : _point( point_ )
 {
 }
 void Magick::densityImage::operator()( Magick::Image &image_ ) const
 {
-  image_.density( _geomery );
+  image_.density( _point );
 }
 
 // Image depth (bits allocated to red/green/blue components)
@@ -1394,7 +1357,7 @@ void Magick::fileNameImage::operator()( Magick::Image &image_ ) const
 }
 
 // Filter to use when resizing image
-Magick::filterTypeImage::filterTypeImage( const FilterTypes filterType_ )
+Magick::filterTypeImage::filterTypeImage( const FilterType filterType_ )
   : _filterType( filterType_ )
 {
 }
@@ -1424,7 +1387,7 @@ void Magick::fontPointsizeImage::operator()( Magick::Image &image_ ) const
 }
 
 // GIF disposal method
-Magick::gifDisposeMethodImage::gifDisposeMethodImage( const size_t disposeMethod_ )
+Magick::gifDisposeMethodImage::gifDisposeMethodImage( const DisposeType disposeMethod_ )
   : _disposeMethod( disposeMethod_ )
 {
 }
@@ -1443,16 +1406,6 @@ void Magick::interlaceTypeImage::operator()( Magick::Image &image_ ) const
   image_.interlaceType( _interlace );
 }
 
-// Linewidth for drawing vector objects (default one)
-Magick::lineWidthImage::lineWidthImage( const double lineWidth_ )
-  : _lineWidth( lineWidth_ )
-{
-}
-void Magick::lineWidthImage::operator()( Magick::Image &image_ ) const
-{
-  image_.lineWidth( _lineWidth );
-}
-
 // File type magick identifier (.e.g "GIF")
 Magick::magickImage::magickImage( const std::string &magick_ )
   : _magick( magick_ )
@@ -1464,13 +1417,13 @@ void Magick::magickImage::operator()( Magick::Image &image_ ) const
 }
 
 // Image supports transparent color
-Magick::matteImage::matteImage( const bool matteFlag_ )
-  : _matteFlag( matteFlag_ )
+Magick::alphaFlagImage::alphaFlagImage( const bool alphaFlag_ )
+  : _alphaFlag( alphaFlag_ )
 {
 }
-void Magick::matteImage::operator()( Magick::Image &image_ ) const
+void Magick::alphaFlagImage::operator()( Magick::Image &image_ ) const
 {
-  image_.matte( _matteFlag );
+  image_.alpha( _alphaFlag );
 }
 
 // Transparent color
@@ -1493,26 +1446,6 @@ void Magick::monochromeImage::operator()( Magick::Image &image_ ) const
   image_.monochrome( _monochromeFlag );
 }
 
-// Pen color
-Magick::penColorImage::penColorImage( const Color &penColor_ )
-  : _penColor( penColor_ )
-{
-}
-void Magick::penColorImage::operator()( Magick::Image &image_ ) const
-{
-  image_.penColor( _penColor );
-}
-
-// Pen texture image.
-Magick::penTextureImage::penTextureImage( const Image &penTexture_ )
-  : _penTexture( penTexture_ )
-{
-}
-void Magick::penTextureImage::operator()( Magick::Image &image_ ) const
-{
-  image_.penTexture( _penTexture );
-}
-
 // Set pixel color at location x & y.
 Magick::pixelColorImage::pixelColorImage( const ssize_t x_,
                                           const ssize_t y_,
@@ -1664,14 +1597,14 @@ void Magick::subRangeImage::operator()( Magick::Image &image_ ) const
   image_.subRange( _subRange );
 }
 
-// Tile name
-Magick::tileNameImage::tileNameImage( const std::string &tileName_ )
-  : _tileName( tileName_ )
+// Anti-alias Postscript and TrueType fonts (default true)
+Magick::textAntiAliasImage::textAntiAliasImage( const bool flag_ )
+  : _flag( flag_ )
 {
 }
-void Magick::tileNameImage::operator()( Magick::Image &image_ ) const
+void Magick::textAntiAliasImage::operator()( Magick::Image &image_ ) const
 {
-  image_.tileName( _tileName );
+  image_.textAntiAlias( _flag );
 }
 
 // Image storage type
@@ -1694,15 +1627,6 @@ void Magick::verboseImage::operator()( Magick::Image &image_ ) const
   image_.verbose( _verbose );
 }
 
-// FlashPix viewing parameters
-Magick::viewImage::viewImage( const std::string &view_ )
-  : _view( view_ ) { }
-
-void Magick::viewImage::operator()( Magick::Image &image_ ) const
-{
-  image_.view( _view );
-}
-
 // X11 display to display to, obtain fonts from, or to capture image
 // from
 Magick::x11DisplayImage::x11DisplayImage( const std::string &display_ )
