@@ -17,7 +17,7 @@
 %                                 July 1999                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -35,41 +35,43 @@
 %
 %
 */
+
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/constitute.h"
-#include "magick/delegate.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/geometry.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/layer.h"
-#include "magick/list.h"
-#include "magick/log.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/opencl.h"
-#include "magick/option.h"
-#include "magick/pixel-accessor.h"
-#include "magick/profile.h"
-#include "magick/property.h"
-#include "magick/quantum-private.h"
-#include "magick/resource_.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/string-private.h"
-#include "magick/module.h"
-#include "magick/transform.h"
-#include "magick/utility.h"
-#include "magick/utility-private.h"
-#include "magick/xml-tree.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/constitute.h"
+#include "MagickCore/delegate.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/geometry.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/layer.h"
+#include "MagickCore/list.h"
+#include "MagickCore/log.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/opencl.h"
+#include "MagickCore/option.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/profile-private.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/string-private.h"
+#include "MagickCore/module.h"
+#include "MagickCore/transform.h"
+#include "MagickCore/utility.h"
+#include "MagickCore/utility-private.h"
+#include "MagickCore/xml-tree.h"
+#include "MagickCore/xml-tree-private.h"
 #if defined(MAGICKCORE_RAW_R_DELEGATE)
 #include <libraw.h>
 #endif
@@ -105,43 +107,45 @@
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(MAGICKCORE_OPENCL_SUPPORT)
 static void InitializeDcrawOpenCL(ExceptionInfo *exception)
 {
-  MagickCLEnv
-    clEnv;
+  MagickCLDevice
+    *devices;
+
+  size_t
+    length;
+
+  ssize_t
+    i;
 
   (void) SetEnvironmentVariable("DCR_CL_PLATFORM",NULL);
   (void) SetEnvironmentVariable("DCR_CL_DEVICE",NULL);
   (void) SetEnvironmentVariable("DCR_CL_DISABLED",NULL);
-  clEnv=GetDefaultOpenCLEnv();
-  if (InitOpenCLEnv(clEnv,exception) != MagickFalse)
+  if (GetOpenCLEnabled() == MagickFalse)
     {
-      char
-        *name;
-
-      MagickBooleanType
-        opencl_disabled;
-
-      GetMagickOpenCLEnvParam(clEnv,MAGICK_OPENCL_ENV_PARAM_OPENCL_DISABLED,
-        sizeof(MagickBooleanType),&opencl_disabled,exception);
-      if (opencl_disabled != MagickFalse)
-        {
-          (void)SetEnvironmentVariable("DCR_CL_DISABLED","1");
-          return;
-        }
-      GetMagickOpenCLEnvParam(clEnv,MAGICK_OPENCL_ENV_PARAM_PLATFORM_VENDOR,
-        sizeof(char *),&name,exception);
-      if (name != (char *) NULL)
-      {
-        (void) SetEnvironmentVariable("DCR_CL_PLATFORM",name);
-        name=RelinquishMagickMemory(name);
-      }
-      GetMagickOpenCLEnvParam(clEnv,MAGICK_OPENCL_ENV_PARAM_DEVICE_NAME,
-        sizeof(char *),&name,exception);
-      if (name != (char *) NULL)
-      {
-        (void) SetEnvironmentVariable("DCR_CL_DEVICE",name);
-        name=RelinquishMagickMemory(name);
-      }
+      (void) SetEnvironmentVariable("DCR_CL_DISABLED","1");
+      return;
     }
+  devices=GetOpenCLDevices(&length,exception);
+  if (devices == (MagickCLDevice *) NULL)
+    return;
+  for (i=0; i < (ssize_t) length; i++)
+  {
+    const char
+      *name;
+
+    MagickCLDevice
+      device;
+
+    device=devices[i];
+    if (GetOpenCLDeviceEnabled(device) == MagickFalse)
+      continue;
+    name=GetOpenCLDeviceVendorName(device);
+    if (name != (const char *) NULL)
+      (void) SetEnvironmentVariable("DCR_CL_PLATFORM",name);
+    name=GetOpenCLDeviceName(device);
+    if (name != (const char *) NULL)
+      (void) SetEnvironmentVariable("DCR_CL_DEVICE",name);
+    return;
+  }
 }
 #else
 static void InitializeDcrawOpenCL(ExceptionInfo *magick_unused(exception))
@@ -160,20 +164,22 @@ static void SetDNGProperties(Image *image,const libraw_data_t *raw_info,
   char
     timestamp[MagickTimeExtent];
 
-  (void) SetImageProperty(image,"dng:make",raw_info->idata.make);
-  (void) SetImageProperty(image,"dng:camera.model.name",raw_info->idata.model);
+  (void) SetImageProperty(image,"dng:make",raw_info->idata.make,exception);
+  (void) SetImageProperty(image,"dng:camera.model.name",raw_info->idata.model,
+    exception);
   (void) FormatMagickTime(raw_info->other.timestamp,sizeof(timestamp),
     timestamp);
-  (void) SetImageProperty(image,"dng:create.date",timestamp);
+  (void) SetImageProperty(image,"dng:create.date",timestamp,exception);
   (void) FormatImageProperty(image,"dng:iso.setting","%.0g",
     (double) raw_info->other.iso_speed);
 #if LIBRAW_COMPILE_CHECK_VERSION_NOTLESS(0,18)
-  (void) SetImageProperty(image,"dng:software",raw_info->idata.software);
+  (void) SetImageProperty(image,"dng:software",raw_info->idata.software,
+    exception);
   if (*raw_info->shootinginfo.BodySerial != '\0')
     (void) SetImageProperty(image,"dng:serial.number",
-      raw_info->shootinginfo.BodySerial);
-  (void) FormatImageProperty(image,"dng:exposure.time","1/%.0f",
-    PerceptibleReciprocal(raw_info->other.shutter));
+      raw_info->shootinginfo.BodySerial,exception);
+  (void) FormatImageProperty(image,"dng:exposure.time","1/%.0g",
+    (double) PerceptibleReciprocal(raw_info->other.shutter));
   (void) FormatImageProperty(image,"dng:f.number","%0.1g",
     (double) raw_info->other.aperture);
   (void) FormatImageProperty(image,"dng:max.aperture.value","%0.1g",
@@ -184,7 +190,7 @@ static void SetDNGProperties(Image *image,const libraw_data_t *raw_info,
     (double) raw_info->color.cam_mul[0],(double) raw_info->color.cam_mul[2],
     (double) raw_info->color.cam_mul[1],(double) raw_info->color.cam_mul[3]);
   (void) SetImageProperty(image,"dng:lens.type",
-    raw_info->lens.makernotes.LensFeatures_suf);
+    raw_info->lens.makernotes.LensFeatures_suf,exception);
   (void) FormatImageProperty(image,"dng:lens","%0.1g-%0.1gmm f/%0.1g-%0.1g",
     (double) raw_info->lens.makernotes.MinFocal,
     (double) raw_info->lens.makernotes.MaxFocal,
@@ -229,93 +235,33 @@ static Image *InvokeDNGDelegate(const ImageInfo *image_info,Image *image,
   ImageInfo
     *read_info;
 
+  MagickBooleanType
+    status;
+
   /*
-    Convert DNG to PPM with delegate.
+    Convert DNG to TIFF with delegate program.
   */
   (void) DestroyImageList(image);
   InitializeDcrawOpenCL(exception);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   read_info=CloneImageInfo(image_info);
   SetImageInfoBlob(read_info,(void *) NULL,0);
-  (void) InvokeDelegate(read_info,image,"dng:decode",(char *) NULL,exception);
+  status=InvokeDelegate(read_info,image,"dng:decode",(char *) NULL,exception);
   image=DestroyImage(image);
-  (void) FormatLocaleString(read_info->filename,MagickPathExtent,"%s.png",
+  if (status == MagickFalse)
+    {
+      read_info=DestroyImageInfo(read_info);
+      return(image);
+    }
+  *read_info->magick='\0';
+  (void) FormatLocaleString(read_info->filename,MagickPathExtent,"%s.tif",
     read_info->unique);
   sans_exception=AcquireExceptionInfo();
   image=ReadImage(read_info,sans_exception);
   sans_exception=DestroyExceptionInfo(sans_exception);
-  if (image == (Image *) NULL)
-    {
-      (void) FormatLocaleString(read_info->filename,MagickPathExtent,"%s.ppm",
-        read_info->unique);
-      image=ReadImage(read_info,exception);
-    }
-  (void) RelinquishUniqueFileResource(read_info->filename);
   if (image != (Image *) NULL)
-    {
-      char
-        filename[MagickPathExtent],
-        *xml;
-
-      ExceptionInfo
-        *sans;
-
-      (void) CopyMagickString(image->magick,read_info->magick,
-        MagickPathExtent);
-      (void) FormatLocaleString(filename,MagickPathExtent,"%s.ufraw",
-        read_info->unique);
-      sans=AcquireExceptionInfo();
-      xml=FileToString(filename,MagickPathExtent,sans);
-      (void) RelinquishUniqueFileResource(filename);
-      if (xml != (char *) NULL)
-        {
-          XMLTreeInfo
-            *ufraw;
-
-          /*
-            Inject.
-          */
-          ufraw=NewXMLTree(xml,sans);
-          if (ufraw != (XMLTreeInfo *) NULL)
-            {
-              char
-                *content,
-                property[MagickPathExtent];
-
-              const char
-                *tag;
-
-              XMLTreeInfo
-                *next;
-
-              if (image->properties == (void *) NULL)
-                image->properties=NewSplayTree(CompareSplayTreeString,
-                  RelinquishMagickMemory,RelinquishMagickMemory);
-              next=GetXMLTreeChild(ufraw,(const char *) NULL);
-              while (next != (XMLTreeInfo *) NULL)
-              {
-                tag=GetXMLTreeTag(next);
-                if (tag == (char *) NULL)
-                  tag="unknown";
-                (void) FormatLocaleString(property,MagickPathExtent,"dng:%s",
-                  tag);
-                content=ConstantString(GetXMLTreeContent(next));
-                (void) StripString(content);
-                if ((LocaleCompare(tag,"log") != 0) &&
-                    (LocaleCompare(tag,"InputFilename") != 0) &&
-                    (LocaleCompare(tag,"OutputFilename") != 0) &&
-                    (LocaleCompare(tag,"OutputType") != 0) &&
-                    (strlen(content) != 0))
-                  (void) AddValueToSplayTree((SplayTreeInfo *)
-                    image->properties,ConstantString(property),content);
-                next=GetXMLTreeSibling(next);
-              }
-              ufraw=DestroyXMLTree(ufraw);
-            }
-          xml=DestroyString(xml);
-        }
-      sans=DestroyExceptionInfo(sans);
-    }
+    (void) CopyMagickString(image->magick,read_info->magick,MagickPathExtent);
+  (void) RelinquishUniqueFileResource(read_info->filename);
   read_info=DestroyImageInfo(read_info);
   return(image);
 }
@@ -327,6 +273,21 @@ static void SetLibRawParams(const ImageInfo *image_info,Image *image,
   const char
     *option;
 
+#if LIBRAW_COMPILE_CHECK_VERSION_NOTLESS(0,20)
+#if LIBRAW_COMPILE_CHECK_VERSION_NOTLESS(0,21)
+  raw_info->rawparams.max_raw_memory_mb=8192;
+#else
+  raw_info->params.max_raw_memory_mb=8192;
+#endif
+  option=GetImageOption(image_info,"dng:max-raw-memory");
+  if (option != (const char *) NULL)
+#if LIBRAW_COMPILE_CHECK_VERSION_NOTLESS(0,21)
+    raw_info->rawparams.max_raw_memory_mb=(unsigned int)
+      StringToInteger(option);
+#else
+   raw_info->params.max_raw_memory_mb=(unsigned int) StringToInteger(option);
+#endif
+#endif
   raw_info->params.user_flip=0;
   raw_info->params.output_bps=16;
   raw_info->params.use_camera_wb=1;
@@ -413,24 +374,25 @@ static void ReadLibRawThumbnail(const ImageInfo *image_info,Image *image,
         *profile;
 
       if (thumbnail->type == LIBRAW_IMAGE_JPEG)
-        SetImageProperty(image,"dng:thumbnail.type","jpeg");
+        SetImageProperty(image,"dng:thumbnail.type","jpeg",exception);
       else if (thumbnail->type == LIBRAW_IMAGE_BITMAP)
         {
           char
             value[15];
 
-          SetImageProperty(image,"dng:thumbnail.type","bitmap");
+          SetImageProperty(image,"dng:thumbnail.type","bitmap",exception);
           (void) FormatLocaleString(value,sizeof(value),"%hu",thumbnail->bits);
-          SetImageProperty(image,"dng:thumbnail.bits",value);
+          SetImageProperty(image,"dng:thumbnail.bits",value,exception);
           (void) FormatLocaleString(value,sizeof(value),"%hu",
             thumbnail->colors);
-          SetImageProperty(image,"dng:thumbnail.colors",value);
+          SetImageProperty(image,"dng:thumbnail.colors",value,exception);
           (void) FormatLocaleString(value,sizeof(value),"%hux%hu",
             thumbnail->width,thumbnail->height);
-          SetImageProperty(image,"dng:thumbnail.geometry",value);
+          SetImageProperty(image,"dng:thumbnail.geometry",value,exception);
         }
-      profile=BlobToStringInfo(thumbnail->data,thumbnail->data_size);
-      (void) SetImageProfile(image,"dng:thumbnail",profile);
+      profile=BlobToProfileStringInfo("dng:thumbnail",thumbnail->data,
+        thumbnail->data_size,exception);
+      (void) SetImageProfilePrivate(image,profile,exception);
     }
   if (thumbnail != (libraw_processed_image_t *) NULL)
     libraw_dcraw_clear_mem(thumbnail);
@@ -454,6 +416,7 @@ static OrientationType LibRawFlipToOrientation(int flip)
     }
   }
 }
+
 #endif
 
 static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
@@ -474,7 +437,7 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -521,7 +484,7 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
         return(DestroyImageList(image));
       }
     libraw_set_dataerror_handler(raw_info,LibRawDataError,exception);
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(_MSC_VER) && (_MSC_VER > 1310)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
     {
       wchar_t
         *path;
@@ -551,12 +514,13 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->page.y=raw_info->sizes.top_margin;
     image->orientation=LibRawFlipToOrientation(raw_info->sizes.flip);
     ReadLibRawThumbnail(image_info,image,raw_info,exception);
+    SetDNGProperties(image,raw_info,exception);
     if (image_info->ping != MagickFalse)
       {
         libraw_close(raw_info);
         return(image);
       }
-    status=SetImageExtent(image,image->columns,image->rows);
+    status=SetImageExtent(image,image->columns,image->rows,exception);
     if (status == MagickFalse)
       {
         libraw_close(raw_info);
@@ -580,10 +544,10 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
         return(DestroyImageList(image));
       }
     raw_image=libraw_dcraw_make_mem_image(raw_info,&errcode);
-    if ((errcode != LIBRAW_SUCCESS) || 
+    if ((errcode != LIBRAW_SUCCESS) ||
         (raw_image == (libraw_processed_image_t *) NULL) ||
         (raw_image->type != LIBRAW_IMAGE_BITMAP) || (raw_image->bits != 16) ||
-        (raw_image->colors < 3) || (raw_image->colors > 4))
+        (raw_image->colors < 1) || (raw_image->colors > 4))
       {
         if (raw_image != (libraw_processed_image_t *) NULL)
           libraw_dcraw_clear_mem(raw_image);
@@ -595,16 +559,12 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (raw_image->colors < 3)
       {
         image->colorspace=GRAYColorspace;
-        image->type=raw_image->colors == 1 ? GrayscaleType : GrayscaleMatteType;
+        image->type=raw_image->colors == 1 ? GrayscaleType : GrayscaleAlphaType;
       }
     image->columns=raw_image->width;
     image->rows=raw_image->height;
     image->depth=raw_image->bits;
-    image->page.width=raw_info->sizes.width;
-    image->page.height=raw_info->sizes.height;
-    image->page.x=raw_info->sizes.left_margin;
-    image->page.y=raw_info->sizes.top_margin;
-    status=SetImageExtent(image,image->columns,image->rows);
+    status=SetImageExtent(image,image->columns,image->rows,exception);
     if (status == MagickFalse)
       {
         libraw_dcraw_clear_mem(raw_image);
@@ -614,23 +574,26 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
     p=(unsigned short *) raw_image->data;
     for (y=0; y < (ssize_t) image->rows; y++)
     {
-      PixelPacket
+      Quantum
         *q;
 
       ssize_t
         x;
 
       q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-      if (q == (PixelPacket *) NULL)
+      if (q == (Quantum *) NULL)
         break;
       for (x=0; x < (ssize_t) image->columns; x++)
       {
-        SetPixelRed(q,ScaleShortToQuantum(*p++));
-        SetPixelGreen(q,ScaleShortToQuantum(*p++));
-        SetPixelBlue(q,ScaleShortToQuantum(*p++));
-        if (raw_image->colors > 3)
-          SetPixelAlpha(q,ScaleShortToQuantum(*p++));
-        q++;
+        SetPixelRed(image,ScaleShortToQuantum(*p++),q);
+        if (raw_image->colors > 2)
+          {
+            SetPixelGreen(image,ScaleShortToQuantum(*p++),q);
+            SetPixelBlue(image,ScaleShortToQuantum(*p++),q);
+          }
+        if ((raw_image->colors) == 2 || (raw_image->colors > 3))
+          SetPixelAlpha(image,ScaleShortToQuantum(*p++),q);
+        q+=(ptrdiff_t) GetPixelChannels(image);
       }
       if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
@@ -648,27 +611,18 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     if (raw_info->color.profile != NULL)
       {
-        profile=BlobToStringInfo(raw_info->color.profile,
-          raw_info->color.profile_length);
-        if (profile != (StringInfo *) NULL)
-          {
-            SetImageProfile(image,"ICC",profile);
-            profile=DestroyStringInfo(profile);
-          }
+        profile=BlobToProfileStringInfo("icc",raw_info->color.profile,
+          raw_info->color.profile_length,exception);
+        (void) SetImageProfilePrivate(image,profile,exception);
       }
 #if LIBRAW_COMPILE_CHECK_VERSION_NOTLESS(0,18)
     if (raw_info->idata.xmpdata != NULL)
       {
-        profile=BlobToStringInfo(raw_info->idata.xmpdata,
-          raw_info->idata.xmplen);
-        if (profile != (StringInfo *) NULL)
-          {
-            SetImageProfile(image,"XMP",profile);
-            profile=DestroyStringInfo(profile);
-          }
+        profile=BlobToProfileStringInfo("xmp",raw_info->idata.xmpdata,
+          raw_info->idata.xmplen,exception);
+        (void) SetImageProfilePrivate(image,profile,exception);
       }
 #endif
-    SetDNGProperties(image,raw_info,exception);
     libraw_close(raw_info);
     return(image);
   }
@@ -700,235 +654,62 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %      size_t RegisterDNGImage(void)
 %
 */
-ModuleExport size_t RegisterDNGImage(void)
+static inline void RegisterDNGMagickInfo(const char *name,
+  const char *description,const char *version)
 {
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("3FR");
+  entry=AcquireMagickInfo("DNG",name,description);
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Hasselblad CFV/H3D39II");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("ARW");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Sony Alpha Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("DNG");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Digital Negative");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("CR2");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Canon Digital Camera Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("CR3");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Canon Digital Camera Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("CRW");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Canon Digital Camera Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("DCR");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Kodak Digital Camera Raw Image File");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("DCR");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Raw Photo Decoder (dcraw)");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("ERF");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Epson Raw Format");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("IIQ");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Phase One Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("KDC");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Kodak Digital Camera Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("K25");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Kodak Digital Camera Raw Image Format");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("MEF");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Mamiya Raw Image File");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("MRW");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Sony (Minolta) Raw Image File");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("NEF");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Nikon Digital SLR Camera Raw Image File");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("NRW");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Nikon Digital SLR Camera Raw Image File");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("ORF");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Olympus Digital Camera Raw Image File");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("PEF");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Pentax Electronic File");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("RAF");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Fuji CCD-RAW Graphic File");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("RAW");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Raw");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("RMF");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Raw Media Format");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("RW2");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Panasonic Lumix Raw Image");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("SRF");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Sony Raw Format");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("SR2");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Sony Raw Format 2");
-  entry->magick_module=ConstantString("DNG");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("X3F");
-  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->format_type=ExplicitFormatType;
-  entry->description=ConstantString("Sigma Camera RAW Picture File");
-  entry->magick_module=ConstantString("DNG");
+  if (*version != '\0')
+    entry->version=ConstantString(version);
   (void) RegisterMagickInfo(entry);
+}
+
+ModuleExport size_t RegisterDNGImage(void)
+{
+  char
+    version[MagickPathExtent];
+
+  *version='\0';
+#if defined(MAGICKCORE_RAW_R_DELEGATE)
+  (void) CopyMagickString(version,libraw_version(),MagickPathExtent);
+#endif
+  RegisterDNGMagickInfo("3FR","Hasselblad CFV/H3D39II Raw Format",version);
+  RegisterDNGMagickInfo("ARW","Sony Alpha Raw Format",version);
+  RegisterDNGMagickInfo("CR2","Canon Digital Camera Raw Format",version);
+  RegisterDNGMagickInfo("CR3","Canon Digital Camera Raw Format",version);
+  RegisterDNGMagickInfo("CRW","Canon Digital Camera Raw Format",version);
+  RegisterDNGMagickInfo("DCR","Kodak Digital Camera Raw Format",version);
+  RegisterDNGMagickInfo("DCRAW","Raw Photo Decoder (dcraw)",version);
+  RegisterDNGMagickInfo("DNG","Digital Negative Raw Format",version);
+  RegisterDNGMagickInfo("ERF","Epson Raw Format",version);
+  RegisterDNGMagickInfo("FFF","Hasselblad CFV/H3D39II Raw Format",version);
+  RegisterDNGMagickInfo("IIQ","Phase One Raw Format",version);
+  RegisterDNGMagickInfo("K25","Kodak Digital Camera Raw Format",version);
+  RegisterDNGMagickInfo("KDC","Kodak Digital Camera Raw Format",version);
+  RegisterDNGMagickInfo("MDC","Minolta Digital Camera Raw Format",version);
+  RegisterDNGMagickInfo("MEF","Mamiya Raw Format",version);
+  RegisterDNGMagickInfo("MOS","Aptus Leaf Raw Format",version);
+  RegisterDNGMagickInfo("MRW","Sony (Minolta) Raw Format",version);
+  RegisterDNGMagickInfo("NEF","Nikon Digital SLR Camera Raw Format",version);
+  RegisterDNGMagickInfo("NRW","Nikon Digital SLR Camera Raw Format",version);
+  RegisterDNGMagickInfo("ORF","Olympus Digital Camera Raw Format",version);
+  RegisterDNGMagickInfo("PEF","Pentax Electronic Raw Format",version);
+  RegisterDNGMagickInfo("RAF","Fuji CCD-RAW Graphic Raw Format",version);
+  RegisterDNGMagickInfo("RAW","Raw",version);
+  RegisterDNGMagickInfo("RMF","Raw Media Format",version);
+  RegisterDNGMagickInfo("RW2","Panasonic Lumix Raw Format",version);
+  RegisterDNGMagickInfo("RWL","Leica Raw Format",version);
+  RegisterDNGMagickInfo("SR2","Sony Raw Format 2",version);
+  RegisterDNGMagickInfo("SRF","Sony Raw Format",version);
+  RegisterDNGMagickInfo("SRW","Samsung Raw Format",version);
+  RegisterDNGMagickInfo("STI","Sinar CaptureShop Raw Format",version);
+  RegisterDNGMagickInfo("X3F","Sigma Camera RAW Format",version);
   return(MagickImageCoderSignature);
 }
 
@@ -954,28 +735,33 @@ ModuleExport size_t RegisterDNGImage(void)
 ModuleExport void UnregisterDNGImage(void)
 {
   (void) UnregisterMagickInfo("X3F");
-  (void) UnregisterMagickInfo("SR2");
+  (void) UnregisterMagickInfo("STI");
+  (void) UnregisterMagickInfo("SRW");
   (void) UnregisterMagickInfo("SRF");
+  (void) UnregisterMagickInfo("SR2");
+  (void) UnregisterMagickInfo("RWL");
   (void) UnregisterMagickInfo("RW2");
   (void) UnregisterMagickInfo("RMF");
-  (void) UnregisterMagickInfo("RAW");
   (void) UnregisterMagickInfo("RAF");
   (void) UnregisterMagickInfo("PEF");
   (void) UnregisterMagickInfo("ORF");
   (void) UnregisterMagickInfo("NRW");
   (void) UnregisterMagickInfo("NEF");
   (void) UnregisterMagickInfo("MRW");
+  (void) UnregisterMagickInfo("MOS");
   (void) UnregisterMagickInfo("MEF");
-  (void) UnregisterMagickInfo("K25");
+  (void) UnregisterMagickInfo("MDC");
   (void) UnregisterMagickInfo("KDC");
+  (void) UnregisterMagickInfo("K25");
   (void) UnregisterMagickInfo("IIQ");
+  (void) UnregisterMagickInfo("FFF");
   (void) UnregisterMagickInfo("ERF");
+  (void) UnregisterMagickInfo("DNG");
+  (void) UnregisterMagickInfo("DCRAW");
   (void) UnregisterMagickInfo("DCR");
   (void) UnregisterMagickInfo("CRW");
   (void) UnregisterMagickInfo("CR3");
   (void) UnregisterMagickInfo("CR2");
-  (void) UnregisterMagickInfo("DNG");
-  (void) UnregisterMagickInfo("DCRAW");
   (void) UnregisterMagickInfo("ARW");
   (void) UnregisterMagickInfo("3FR");
 }
