@@ -10,7 +10,7 @@
 %                            EEEEE  M   M  F                                  %
 %                                                                             %
 %                                                                             %
-%                  Read Windows Enahanced Metafile Format                     %
+%                   Read Windows Enhanced Metafile Format                     %
 %                                                                             %
 %                              Software Design                                %
 %                              Bill Radcliffe                                 %
@@ -18,7 +18,7 @@
 %                               Dirk Lemstra                                  %
 %                               January 2014                                  %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -39,7 +39,7 @@
  * Include declarations.
  */
 
-#include "magick/studio.h"
+#include "MagickCore/studio.h"
 #if defined(MAGICKCORE_WINGDI32_DELEGATE)
 #  if !defined(_MSC_VER)
 #    if defined(__CYGWIN__)
@@ -48,27 +48,31 @@
 #      include <wingdi.h>
 #    endif
 #  else
+#pragma warning(disable: 4458)
 #    include <gdiplus.h>
+#pragma warning(default: 4458)
 #    pragma comment(lib, "gdiplus.lib")
 #  endif
 #endif
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/geometry.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/pixel.h"
-#include "magick/pixel-accessor.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/geometry.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/pixel.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/utility-private.h"
+#include "coders/emf.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -411,7 +415,8 @@ static HENHMETAFILE ReadEnhMetaFile(const char *path,ssize_t *width,
     }
   ReadFile(hFile,pBits,dwSize,&dwSize,NULL);
   CloseHandle(hFile);
-  if (((PAPMHEADER) pBits)->dwKey != 0x9ac6cdd7l)
+  if ((((PAPMHEADER) pBits)->dwKey != 0x9ac6cdd7l) ||
+      (((PAPMHEADER) pBits)->wInch == 0))
     {
       pBits=(BYTE *) DestroyString((char *) pBits);
       return((HENHMETAFILE) NULL);
@@ -463,7 +468,7 @@ static Image *ReadEMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ssize_t
     x;
 
-  PixelPacket
+  Quantum
     *q;
 
   RGBQUAD
@@ -475,7 +480,7 @@ static Image *ReadEMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     width,
     y;
 
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   hemf=ReadEnhMetaFile(image_info->filename,&width,&height);
   if (hemf == (HENHMETAFILE) NULL)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
@@ -487,15 +492,15 @@ static Image *ReadEMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
       y_resolution=DefaultResolution;
       x_resolution=DefaultResolution;
-      if (image->y_resolution > 0)
+      if (image->resolution.y > 0)
         {
-          y_resolution=image->y_resolution;
+          y_resolution=image->resolution.y;
           if (image->units == PixelsPerCentimeterResolution)
             y_resolution*=CENTIMETERS_INCH;
         }
-      if (image->x_resolution > 0)
+      if (image->resolution.x > 0)
         {
-          x_resolution=image->x_resolution;
+          x_resolution=image->resolution.x;
           if (image->units == PixelsPerCentimeterResolution)
             x_resolution*=CENTIMETERS_INCH;
         }
@@ -510,12 +515,9 @@ static Image *ReadEMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       (void) GetGeometry(image_info->size,(ssize_t *) NULL,(ssize_t *) NULL,
         &image->columns,&image->rows);
     }
-  status=SetImageExtent(image,image->columns,image->rows);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
+    return(DestroyImageList(image));
   if (image_info->page != (char *) NULL)
     {
       char
@@ -536,22 +538,22 @@ static Image *ReadEMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         {
           flags=ParseMetaGeometry(geometry,&sans,&sans,&image->columns,
             &image->rows);
-          if (image->x_resolution != 0.0)
-            image->columns=(size_t) floor((image->columns*image->x_resolution)+
+          if (image->resolution.x != 0.0)
+            image->columns=(size_t) floor((image->columns*image->resolution.x)+
               0.5);
-          if (image->y_resolution != 0.0)
-            image->rows=(size_t) floor((image->rows*image->y_resolution)+0.5);
+          if (image->resolution.y != 0.0)
+            image->rows=(size_t) floor((image->rows*image->resolution.y)+0.5);
         }
       else
         {
           *p='\0';
           flags=ParseMetaGeometry(geometry,&sans,&sans,&image->columns,
             &image->rows);
-          if (image->x_resolution != 0.0)
-            image->columns=(size_t) floor(((image->columns*image->x_resolution)/
+          if (image->resolution.x != 0.0)
+            image->columns=(size_t) floor(((image->columns*image->resolution.x)/
               DefaultResolution)+0.5);
-          if (image->y_resolution != 0.0)
-            image->rows=(size_t) floor(((image->rows*image->y_resolution)/
+          if (image->resolution.y != 0.0)
+            image->rows=(size_t) floor(((image->rows*image->resolution.y)/
               DefaultResolution)+0.5);
         }
       (void) flags;
@@ -622,16 +624,16 @@ static Image *ReadEMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-    if (q == (PixelPacket *) NULL)
+    if (q == (Quantum *) NULL)
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      SetPixelRed(q,ScaleCharToQuantum(pBits->rgbRed));
-      SetPixelGreen(q,ScaleCharToQuantum(pBits->rgbGreen));
-      SetPixelBlue(q,ScaleCharToQuantum(pBits->rgbBlue));
-      SetPixelOpacity(q,OpaqueOpacity);
+      SetPixelRed(image,ScaleCharToQuantum(pBits->rgbRed),q);
+      SetPixelGreen(image,ScaleCharToQuantum(pBits->rgbGreen),q);
+      SetPixelBlue(image,ScaleCharToQuantum(pBits->rgbBlue),q);
+      SetPixelAlpha(image,OpaqueAlpha,q);
       pBits++;
-      q++;
+      q+=(ptrdiff_t) GetPixelChannels(image);
     }
     if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
@@ -646,13 +648,13 @@ static Image *ReadEMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
 static inline void EMFSetDimensions(Image * image,Gdiplus::Image *source)
 {
-  if ((image->x_resolution <= 0.0) || (image->y_resolution <= 0.0))
+  if ((image->resolution.x <= 0.0) || (image->resolution.y <= 0.0))
     return;
 
   image->columns=(size_t) floor((Gdiplus::REAL) source->GetWidth()/
-    source->GetHorizontalResolution()*image->x_resolution+0.5);
-  image->rows=(size_t)floor((Gdiplus::REAL) source->GetHeight()/
-    source->GetVerticalResolution()*image->y_resolution+0.5);
+    source->GetHorizontalResolution()*image->resolution.x+0.5);
+  image->rows=(size_t) floor((Gdiplus::REAL) source->GetHeight()/
+    source->GetVerticalResolution()*image->resolution.y+0.5);
 }
 
 static Image *ReadEMFImage(const ImageInfo *image_info,
@@ -685,7 +687,7 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
   MagickStatusType
     flags;
 
-  PixelPacket
+  Quantum
     *q;
 
   ssize_t
@@ -701,96 +703,96 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
     *p;
 
   wchar_t
-    fileName[MaxTextExtent];
+    *path;
 
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
+  assert(exception != (ExceptionInfo *) NULL);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  assert(exception != (ExceptionInfo *) NULL);
-
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   if (Gdiplus::GdiplusStartup(&token,&startup_input,NULL) != 
     Gdiplus::Status::Ok)
     ThrowReaderException(CoderError, "GdiplusStartupFailed");
-  MultiByteToWideChar(CP_UTF8,0,image->filename,-1,fileName,MaxTextExtent);
-  source=Gdiplus::Image::FromFile(fileName);
+  source=(Gdiplus::Image *) NULL;
+  path=create_wchar_path(image->filename);
+  if (path != (wchar_t *) NULL)
+    {
+      source=Gdiplus::Image::FromFile(path);
+      path=(wchar_t *) RelinquishMagickMemory(path);
+    }
   if (source == (Gdiplus::Image *) NULL)
     {
       Gdiplus::GdiplusShutdown(token);
       ThrowReaderException(FileOpenError,"UnableToOpenFile");
     }
-
-  image->x_resolution=source->GetHorizontalResolution();
-  image->y_resolution=source->GetVerticalResolution();
+  image->resolution.x=source->GetHorizontalResolution();
+  image->resolution.y=source->GetVerticalResolution();
   image->columns=(size_t) source->GetWidth();
   image->rows=(size_t) source->GetHeight();
   if (image_info->size != (char *) NULL)
     {
       (void) GetGeometry(image_info->size,(ssize_t *) NULL,(ssize_t *) NULL,
         &image->columns,&image->rows);
-      image->x_resolution=source->GetHorizontalResolution()*image->columns/
+      image->resolution.x=source->GetHorizontalResolution()*image->columns/
         source->GetWidth();
-      image->y_resolution=source->GetVerticalResolution()*image->rows/
+      image->resolution.y=source->GetVerticalResolution()*image->rows/
         source->GetHeight();
-      if (image->x_resolution == 0)
-        image->x_resolution=image->y_resolution;
-      else if (image->y_resolution == 0)
-        image->y_resolution=image->x_resolution;
+      if (image->resolution.x == 0)
+        image->resolution.x=image->resolution.y;
+      else if (image->resolution.y == 0)
+        image->resolution.y=image->resolution.x;
       else
-        image->x_resolution=image->y_resolution=MagickMin(
-          image->x_resolution,image->y_resolution);
+        image->resolution.x=image->resolution.y=MagickMin(
+          image->resolution.x,image->resolution.y);
       EMFSetDimensions(image,source);
     }
   else if (image_info->density != (char *) NULL)
     {
       flags=ParseGeometry(image_info->density,&geometry_info);
-      image->x_resolution=geometry_info.rho;
-      image->y_resolution=geometry_info.sigma;
-      if ((flags & SigmaValue) == 0)
-        image->y_resolution=image->x_resolution;
+      if ((flags & RhoValue) != 0)
+        image->resolution.x=geometry_info.rho;
+      image->resolution.y=image->resolution.x;
+      if ((flags & SigmaValue) != 0)
+        image->resolution.y=geometry_info.sigma;
       EMFSetDimensions(image,source);
     }
-  if (SetImageExtent(image,image->columns,image->rows) == MagickFalse)
+  if (SetImageExtent(image,image->columns,image->rows,exception) == MagickFalse)
     {
       delete source;
       Gdiplus::GdiplusShutdown(token);
-      InheritException(exception,&image->exception);
       return(DestroyImageList(image));
     }
-  image->matte=MagickTrue;
+  image->alpha_trait=BlendPixelTrait;
   if (image->ping != MagickFalse)
     {
       delete source;
       Gdiplus::GdiplusShutdown(token);
       return(image);
     }
-
   bitmap=new Gdiplus::Bitmap((INT) image->columns,(INT) image->rows,
     PixelFormat32bppARGB);
   graphics=Gdiplus::Graphics::FromImage(bitmap);
   graphics->SetInterpolationMode(Gdiplus::InterpolationModeHighQualityBicubic);
   graphics->SetSmoothingMode(Gdiplus::SmoothingModeHighQuality);
   graphics->SetTextRenderingHint(Gdiplus::TextRenderingHintClearTypeGridFit);
-  graphics->Clear(Gdiplus::Color((BYTE) ScaleQuantumToChar(QuantumRange-
-    image->background_color.opacity),(BYTE) ScaleQuantumToChar(
+  graphics->Clear(Gdiplus::Color((BYTE) ScaleQuantumToChar(
+    image->background_color.alpha),(BYTE) ScaleQuantumToChar(
     image->background_color.red),(BYTE) ScaleQuantumToChar(
     image->background_color.green),(BYTE) ScaleQuantumToChar(
     image->background_color.blue)));
   graphics->DrawImage(source,0,0,(INT) image->columns,(INT) image->rows);
   delete graphics;
   delete source;
-
   rect=Gdiplus::Rect(0,0,(INT) image->columns,(INT) image->rows);
   if (bitmap->LockBits(&rect,Gdiplus::ImageLockModeRead,PixelFormat32bppARGB,
-    &bitmap_data) != Gdiplus::Ok)
-  {
-    delete bitmap;
-    Gdiplus::GdiplusShutdown(token);
-    ThrowReaderException(FileOpenError,"UnableToReadImageData");
-  }
-
+        &bitmap_data) != Gdiplus::Ok)
+    {
+      delete bitmap;
+      Gdiplus::GdiplusShutdown(token);
+      ThrowReaderException(FileOpenError,"UnableToReadImageData");
+    }
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     p=(unsigned char *) bitmap_data.Scan0+(y*abs(bitmap_data.Stride));
@@ -798,29 +800,28 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
       q=GetAuthenticPixels(image,0,image->rows-y-1,image->columns,1,exception);
     else
       q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
-    if (q == (PixelPacket *) NULL)
+    if (q == (Quantum *) NULL)
       break;
 
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      SetPixelBlue(q,ScaleCharToQuantum(*p++));
-      SetPixelGreen(q,ScaleCharToQuantum(*p++));
-      SetPixelRed(q,ScaleCharToQuantum(*p++));
-      SetPixelAlpha(q,ScaleCharToQuantum(*p++));
-      q++;
+      SetPixelBlue(image,ScaleCharToQuantum(*p++),q);
+      SetPixelGreen(image,ScaleCharToQuantum(*p++),q);
+      SetPixelRed(image,ScaleCharToQuantum(*p++),q);
+      SetPixelAlpha(image,ScaleCharToQuantum(*p++),q);
+      q+=(ptrdiff_t) GetPixelChannels(image);
     }
 
     if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
   }
-
   bitmap->UnlockBits(&bitmap_data);
   delete bitmap;
   Gdiplus::GdiplusShutdown(token);
   return(image);
 }
 #  endif /* _MSC_VER */
-#endif /* MAGICKCORE_WINGDI32_DELEGATE */
+#endif /* MAGICKCORE_EMF_DELEGATE */
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -850,24 +851,19 @@ ModuleExport size_t RegisterEMFImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("EMF");
+  entry=AcquireMagickInfo("EMF","EMF","Windows Enhanced Meta File");
 #if defined(MAGICKCORE_WINGDI32_DELEGATE)
   entry->decoder=ReadEMFImage;
 #endif
-  entry->description=ConstantString(
-    "Windows Enhanced Meta File");
   entry->magick=(IsImageFormatHandler *) IsEMF;
-  entry->blob_support=MagickFalse;
-  entry->magick_module=ConstantString("EMF");
+  entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("WMF");
+  entry=AcquireMagickInfo("EMF","WMF","Windows Meta File");
 #if defined(MAGICKCORE_WINGDI32_DELEGATE)
   entry->decoder=ReadEMFImage;
 #endif
-  entry->description=ConstantString("Windows Meta File");
   entry->magick=(IsImageFormatHandler *) IsWMF;
-  entry->blob_support=MagickFalse;
-  entry->magick_module=ConstantString("EMF");
+  entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
