@@ -22,7 +22,7 @@
 %                                                                             %
 %                      Copyright 2017-2018 YANDEX LLC.                        %
 %                                                                             %
-%  Copyright @ 2018 ImageMagick Studio LLC, a non-profit organization         %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -44,41 +44,43 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/artifact.h"
-#include "magick/attribute.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/client.h"
-#include "magick/colorspace-private.h"
-#include "magick/property.h"
-#include "magick/display.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/montage.h"
-#include "magick/transform.h"
-#include "magick/distort.h"
-#include "magick/memory_.h"
-#include "magick/memory-private.h"
-#include "magick/option.h"
-#include "magick/pixel-accessor.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/string-private.h"
-#include "magick/module.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/artifact.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/client.h"
+#include "MagickCore/colorspace-private.h"
+#include "MagickCore/property.h"
+#include "MagickCore/display.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/montage.h"
+#include "MagickCore/transform.h"
+#include "MagickCore/distort.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/memory-private.h"
+#include "MagickCore/option.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/profile-private.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/string-private.h"
+#include "MagickCore/module.h"
+#include "MagickCore/utility.h"
+#define HEIC_COMPUTE_NUMERIC_VERSION(major,minor,patch) \
+  (((major) << 24) | ((minor) << 16) | ((patch) << 8) | 0)
 #if defined(MAGICKCORE_HEIC_DELEGATE)
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(__MINGW32__)
-#include <heif.h>
-#else
 #include <libheif/heif.h>
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,17,0)
+#include <libheif/heif_properties.h>
 #endif
 #endif
 
@@ -87,9 +89,9 @@
   Forward declarations.
 */
 static MagickBooleanType
-  WriteHEICImage(const ImageInfo *,Image *);
-
-/*x
+  WriteHEICImage(const ImageInfo *,Image *,ExceptionInfo *);
+
+/*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
@@ -171,12 +173,8 @@ static MagickBooleanType ReadHEICColorProfile(Image *image,
       StringInfo
         *profile;
 
-      profile=BlobToStringInfo(color_profile,length);
-      if (profile != (StringInfo*) NULL)
-        {
-          (void) SetImageProfile(image,"icc",profile);
-          profile=DestroyStringInfo(profile);
-        }
+      profile=BlobToProfileStringInfo("icc",color_profile,length,exception);
+      (void) SetImageProfilePrivate(image,profile,exception);
     }
   color_profile=(unsigned char *) RelinquishMagickMemory(color_profile);
   return(MagickTrue);
@@ -213,7 +211,9 @@ static MagickBooleanType ReadHEICExifProfile(Image *image,
   if ((MagickSizeType) length > GetBlobSize(image))
     ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
       image->filename);
-  exif_profile=AcquireStringInfo(length);
+  exif_profile=AcquireProfileStringInfo("exif",length,exception);
+  if (exif_profile == (StringInfo*) NULL)
+    return(MagickTrue);
   error=heif_image_handle_get_metadata(image_handle,id,
     GetStringInfoDatum(exif_profile));
   if ((IsHEIFSuccess(image,&error,exception) != MagickFalse) && (length > 4))
@@ -252,10 +252,12 @@ static MagickBooleanType ReadHEICExifProfile(Image *image,
       if (offset < GetStringInfoLength(exif_profile))
         {
           (void) DestroyStringInfo(SplitStringInfo(exif_profile,offset));
-          (void) SetImageProfile(image,"exif",exif_profile);
+          (void) SetImageProfilePrivate(image,exif_profile,exception);
+          exif_profile=(StringInfo *) NULL;
         }
     }
-  exif_profile=DestroyStringInfo(exif_profile);
+  if (exif_profile != (StringInfo *) NULL)
+    exif_profile=DestroyStringInfo(exif_profile);
   return(MagickTrue);
 }
 
@@ -299,12 +301,8 @@ static MagickBooleanType ReadHEICXMPProfile(Image *image,
       StringInfo
         *profile;
 
-      profile=BlobToStringInfo(xmp_profile,length);
-      if (profile != (StringInfo*) NULL)
-        {
-          (void) SetImageProfile(image,"xmp",profile);
-          profile=DestroyStringInfo(profile);
-        }
+      profile=BlobToProfileStringInfo("xmp",xmp_profile,length,exception);
+      (void) SetImageProfilePrivate(image,profile,exception);
     }
   xmp_profile=(unsigned char *) RelinquishMagickMemory(xmp_profile);
   return(MagickTrue);
@@ -350,7 +348,7 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
   image->columns=(size_t) heif_image_handle_get_width(image_handle);
   image->rows=(size_t) heif_image_handle_get_height(image_handle);
   if (heif_image_handle_has_alpha_channel(image_handle) != 0)
-    image->matte=MagickTrue;
+    image->alpha_trait=BlendPixelTrait;
   image->depth=8;
   bits_per_pixel=heif_image_handle_get_luma_bits_per_pixel(image_handle);
   if (bits_per_pixel != -1)
@@ -358,7 +356,74 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
   preserve_orientation=IsStringTrue(GetImageOption(image_info,
     "heic:preserve-orientation"));
   if (preserve_orientation == MagickFalse)
-    (void) SetImageProperty(image,"exif:Orientation","1");
+    (void) SetImageProperty(image,"exif:Orientation","1",exception);
+  else
+    {
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,17,0)
+      heif_item_id
+        item_id;
+
+      heif_property_id
+        transforms[1];
+
+      int
+        count;
+
+      struct heif_context
+        *heif_context;
+
+      heif_context=heif_image_handle_get_context(image_handle);
+      item_id=heif_image_handle_get_item_id(image_handle);
+      count=heif_item_get_transformation_properties(heif_context,item_id,
+        transforms,1);
+      if (count == 1)
+        {
+          enum heif_transform_mirror_direction
+            mirror;
+
+          int
+            rotation_ccw;
+
+          mirror=heif_item_get_property_transform_mirror(heif_context,item_id,
+            transforms[0]);
+          rotation_ccw=heif_item_get_property_transform_rotation_ccw(
+            heif_context,item_id,transforms[0]);
+          switch(mirror)
+          {
+            case heif_transform_mirror_direction_horizontal:
+            {
+              if (rotation_ccw == 0)
+                image->orientation=TopRightOrientation;
+              else if (rotation_ccw == 270)
+                image->orientation=LeftTopOrientation;
+              break;
+            }
+            case heif_transform_mirror_direction_vertical:
+            {
+              if (rotation_ccw == 0)
+                image->orientation=BottomLeftOrientation;
+              else if (rotation_ccw == 270)
+                image->orientation=RightBottomOrientation;
+              break;
+            }
+            case heif_transform_mirror_direction_invalid:
+            {
+              if (rotation_ccw == 0)
+                image->orientation=TopLeftOrientation;
+              else if (rotation_ccw == 90)
+                image->orientation=LeftBottomOrientation;
+              else if (rotation_ccw == 180)
+                image->orientation=BottomRightOrientation;
+              else if (rotation_ccw == 270)
+                image->orientation=RightTopOrientation;
+              break;
+            }
+          }
+        }
+#endif
+      image->columns=(size_t) heif_image_handle_get_ispe_width(image_handle);
+      image->rows=(size_t) heif_image_handle_get_ispe_height(image_handle);
+    }
   if (ReadHEICColorProfile(image,image_handle,exception) == MagickFalse)
     return(MagickFalse);
   if (ReadHEICExifProfile(image,image_handle,exception) == MagickFalse)
@@ -369,21 +434,51 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
     return(MagickTrue);
   if (HEICSkipImage(image_info,image) != MagickFalse)
     return(MagickTrue);
-  status=SetImageExtent(image,image->columns,image->rows);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
     return(MagickFalse);
   decode_options=heif_decoding_options_alloc();
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,16,0)
+  {
+    const char
+      *option;
+
+    option=GetImageOption(image_info,"heic:chroma-upsampling");
+    if (option != (char *) NULL)
+      {
+        if (LocaleCompare(option,"nearest-neighbor") == 0)
+          {
+            decode_options->color_conversion_options.
+              only_use_preferred_chroma_algorithm=1;
+            decode_options->color_conversion_options.
+              preferred_chroma_upsampling_algorithm=
+              heif_chroma_upsampling_nearest_neighbor;
+          }
+        else if (LocaleCompare(option,"bilinear") == 0)
+          {
+            decode_options->color_conversion_options.
+              only_use_preferred_chroma_algorithm=1;
+            decode_options->color_conversion_options.
+              preferred_chroma_upsampling_algorithm=
+              heif_chroma_upsampling_bilinear;
+          }
+      }
+    }
+#endif
   if (preserve_orientation != MagickFalse)
     decode_options->ignore_transformations=1;
-  chroma=heif_chroma_interleaved_RGB;
-  if (image->depth > 8)
-    chroma=heif_chroma_interleaved_RRGGBB_LE;
-  if (image->matte != MagickFalse)
+  if (image->alpha_trait != UndefinedPixelTrait)
     {
       chroma=heif_chroma_interleaved_RGBA;
       if (image->depth > 8)
         chroma=heif_chroma_interleaved_RRGGBBAA_LE;
     }
+  else
+    {
+      chroma=heif_chroma_interleaved_RGB;
+      if (image->depth > 8)
+        chroma=heif_chroma_interleaved_RRGGBB_LE;
+    }
   error=heif_decode_image(image_handle,&heif_image,heif_colorspace_RGB,chroma,
     decode_options);
   heif_decoding_options_free(decode_options);
@@ -392,7 +487,7 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
   channel=heif_channel_interleaved;
   image->columns=(size_t) heif_image_get_width(heif_image,channel);
   image->rows=(size_t) heif_image_get_height(heif_image,channel);
-  status=SetImageExtent(image,image->columns,image->rows);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
     {
       heif_image_release(heif_image);
@@ -408,7 +503,7 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
   if (image->depth <= 8)
     for (y=0; y < (ssize_t) image->rows; y++)
     {
-      PixelPacket
+      Quantum
         *q;
 
       ssize_t
@@ -418,17 +513,17 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
         Transform 8-bit image.
       */
       q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-      if (q == (PixelPacket *) NULL)
+      if (q == (Quantum *) NULL)
         break;
       p=pixels+(y*stride);
       for (x=0; x < (ssize_t) image->columns; x++)
       {
-        SetPixelRed(q,ScaleCharToQuantum((unsigned char) *(p++)));
-        SetPixelGreen(q,ScaleCharToQuantum((unsigned char) *(p++)));
-        SetPixelBlue(q,ScaleCharToQuantum((unsigned char) *(p++)));
-        if (image->matte != MagickFalse)
-          SetPixelAlpha(q,ScaleCharToQuantum((unsigned char) *(p++)));
-        q++;
+        SetPixelRed(image,ScaleCharToQuantum((unsigned char) *(p++)),q);
+        SetPixelGreen(image,ScaleCharToQuantum((unsigned char) *(p++)),q);
+        SetPixelBlue(image,ScaleCharToQuantum((unsigned char) *(p++)),q);
+        if (image->alpha_trait != UndefinedPixelTrait)
+          SetPixelAlpha(image,ScaleCharToQuantum((unsigned char) *(p++)),q);
+        q+=(ptrdiff_t) GetPixelChannels(image);
       }
       if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
@@ -436,7 +531,7 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
   else
     for (y=0; y < (ssize_t) image->rows; y++)
     {
-      PixelPacket
+      Quantum
         *q;
 
       ssize_t
@@ -446,24 +541,24 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
         Transform 10-bit or 12-bit image.
       */
       q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-      if (q == (PixelPacket *) NULL)
+      if (q == (Quantum *) NULL)
         break;
       p=pixels+(y*stride);
       for (x=0; x < (ssize_t) image->columns; x++)
       {
         unsigned short pixel = (((unsigned short) *(p+1) << 8) |
-          (*(p+0))) << shift; p+=2;
-        SetPixelRed(q,ScaleShortToQuantum(pixel));
-        pixel=(((unsigned short) *(p+1) << 8) | (*(p+0))) << shift; p+=2;
-        SetPixelGreen(q,ScaleShortToQuantum(pixel));
-        pixel=(((unsigned short) *(p+1) << 8) | (*(p+0))) << shift; p+=2;
-        SetPixelBlue(q,ScaleShortToQuantum(pixel));
-        if (image->matte != MagickFalse)
+          (*(p+0))) << shift; p+=(ptrdiff_t) 2;
+        SetPixelRed(image,ScaleShortToQuantum(pixel),q);
+        pixel=(((unsigned short) *(p+1) << 8) | (*(p+0))) << shift; p+=(ptrdiff_t) 2;
+        SetPixelGreen(image,ScaleShortToQuantum(pixel),q);
+        pixel=(((unsigned short) *(p+1) << 8) | (*(p+0))) << shift; p+=(ptrdiff_t) 2;
+        SetPixelBlue(image,ScaleShortToQuantum(pixel),q);
+        if (image->alpha_trait != UndefinedPixelTrait)
           {
-            pixel=(((unsigned short) *(p+1) << 8) | (*(p+0))) << shift; p+=2;
-            SetPixelAlpha(q,ScaleShortToQuantum(pixel));
+            pixel=(((unsigned short) *(p+1) << 8) | (*(p+0))) << shift; p+=(ptrdiff_t) 2;
+            SetPixelAlpha(image,ScaleShortToQuantum(pixel),q);
           }
-        q++;
+        q+=(ptrdiff_t) GetPixelChannels(image);
       }
       if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
@@ -506,7 +601,7 @@ static void ReadHEICDepthImage(const ImageInfo *image_info,Image *image,
     &depth_handle);
   if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
     return;
-  AcquireNextImage(image_info,image);
+  AcquireNextImage(image_info,image,exception);
   if (GetNextImageInList(image) != (Image *) NULL)
     {
       image=SyncNextImageInList(image);
@@ -527,6 +622,9 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   Image
     *image;
 
+  int
+    max_size;
+
   MagickBooleanType
     status;
 
@@ -543,7 +641,7 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
     *image_handle;
 
   unsigned char
-    magic[12];
+    magic[128];
 
   /*
     Open image file.
@@ -555,7 +653,7 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(DestroyImageList(image));
@@ -565,7 +663,7 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   if (filetype_check == heif_filetype_no)
     ThrowReaderException(CoderError,"ImageTypeNotSupported");
   (void) CloseBlob(image);
-#if LIBHEIF_NUMERIC_VERSION >= 0x010b0000
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,11,0)
   if (heif_has_compatible_brand(magic,sizeof(magic), "avif") == 1)
     (void) CopyMagickString(image->magick,"AVIF",MagickPathExtent);
 #endif
@@ -575,6 +673,10 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   heif_context=heif_context_alloc();
   if (heif_context == (struct heif_context *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  max_size=(int) MagickMin(MagickMin(GetMagickResourceLimit(WidthResource),
+    GetMagickResourceLimit(HeightResource)),INT_MAX);
+  if (max_size != INT_MAX)
+    heif_context_set_maximum_image_size_limit(heif_context,max_size);
   error=heif_context_read_from_file(heif_context,image->filename,
     (const struct heif_reading_options *) NULL);
   if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
@@ -597,7 +699,7 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
     }
   status=ReadHEICImageHandle(image_info,image,image_handle,exception);
   heif_image_handle_release(image_handle);
-  count=(size_t) heif_context_get_number_of_top_level_images(heif_context);
+  count=(ssize_t) heif_context_get_number_of_top_level_images(heif_context);
   if ((status != MagickFalse) && (count > 1))
     {
       heif_item_id
@@ -621,7 +723,7 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
         /*
           Allocate next image structure.
         */
-        AcquireNextImage(image_info,image);
+        AcquireNextImage(image_info,image,exception);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             status=MagickFalse;
@@ -696,6 +798,9 @@ static MagickBooleanType IsHEIC(const unsigned char *magick,const size_t length)
   type=heif_check_filetype(magick,(int) length);
   if (type == heif_filetype_yes_supported)
     return(MagickTrue);
+#else
+  magick_unreferenced(magick);
+  magick_unreferenced(length);
 #endif
   return(MagickFalse);
 }
@@ -727,41 +832,58 @@ ModuleExport size_t RegisterHEICImage(void)
   MagickInfo
     *entry;
 
-#if defined(MAGICKCORE_HEIC_DELEGATE) && (LIBHEIF_NUMERIC_VERSION >= 0x010e0000)
-  heif_init(NULL);
+#if defined(MAGICKCORE_HEIC_DELEGATE)
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,14,0)
+  heif_init((struct heif_init_params *) NULL);
 #endif
-  entry=SetMagickInfo("HEIC");
+#endif
+  entry=AcquireMagickInfo("HEIC","HEIC","High Efficiency Image Format");
 #if defined(MAGICKCORE_HEIC_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadHEICImage;
   if (heif_have_encoder_for_format(heif_compression_HEVC))
     entry->encoder=(EncodeImageHandler *) WriteHEICImage;
 #endif
   entry->magick=(IsImageFormatHandler *) IsHEIC;
-  entry->description=ConstantString("Apple High efficiency Image Format");
   entry->mime_type=ConstantString("image/heic");
 #if defined(LIBHEIF_VERSION)
   entry->version=ConstantString(LIBHEIF_VERSION);
 #endif
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("HEIF");
+  entry=AcquireMagickInfo("HEIC","HEIF","High Efficiency Image Format");
 #if defined(MAGICKCORE_HEIC_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadHEICImage;
   if (heif_have_encoder_for_format(heif_compression_HEVC))
     entry->encoder=(EncodeImageHandler *) WriteHEICImage;
 #endif
   entry->magick=(IsImageFormatHandler *) IsHEIC;
-  entry->description=ConstantString("Apple High efficiency Image Format");
-  entry->mime_type=ConstantString("image/heic");
+  entry->mime_type=ConstantString("image/heif");
 #if defined(LIBHEIF_VERSION)
   entry->version=ConstantString(LIBHEIF_VERSION);
 #endif
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
-#if LIBHEIF_NUMERIC_VERSION > 0x01060200
-  entry=SetMagickInfo("AVIF");
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,7,0)
+  entry=AcquireMagickInfo("HEIC","AVCI","AVC Image File Format");
+#if defined(MAGICKCORE_HEIC_DELEGATE)
+  if (heif_have_decoder_for_format(heif_compression_AVC))
+    entry->decoder=(DecodeImageHandler *) ReadHEICImage;
+  if (heif_have_encoder_for_format(heif_compression_AVC))
+    entry->encoder=(EncodeImageHandler *) WriteHEICImage;
+#endif
+  entry->magick=(IsImageFormatHandler *) IsHEIC;
+  entry->mime_type=ConstantString("image/avci");
+#if defined(LIBHEIF_VERSION)
+  entry->version=ConstantString(LIBHEIF_VERSION);
+#endif
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
+  (void) RegisterMagickInfo(entry);
+#endif
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,7,0)
+  entry=AcquireMagickInfo("HEIC","AVIF","AV1 Image File Format");
 #if defined(MAGICKCORE_HEIC_DELEGATE)
   if (heif_have_decoder_for_format(heif_compression_AV1))
     entry->decoder=(DecodeImageHandler *) ReadHEICImage;
@@ -769,13 +891,12 @@ ModuleExport size_t RegisterHEICImage(void)
     entry->encoder=(EncodeImageHandler *) WriteHEICImage;
 #endif
   entry->magick=(IsImageFormatHandler *) IsHEIC;
-  entry->description=ConstantString("AV1 Image File Format");
   entry->mime_type=ConstantString("image/avif");
 #if defined(LIBHEIF_VERSION)
   entry->version=ConstantString(LIBHEIF_VERSION);
 #endif
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
 #endif
   return(MagickImageCoderSignature);
@@ -802,14 +923,16 @@ ModuleExport size_t RegisterHEICImage(void)
 */
 ModuleExport void UnregisterHEICImage(void)
 {
-#if LIBHEIF_NUMERIC_VERSION > 0x01060200
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,7,0)
   (void) UnregisterMagickInfo("AVIF");
 #endif
   (void) UnregisterMagickInfo("HEIC");
   (void) UnregisterMagickInfo("HEIF");
-#if defined(MAGICKCORE_HEIC_DELEGATE) && (LIBHEIF_NUMERIC_VERSION >= 0x010e0000)
+#if defined(MAGICKCORE_HEIC_DELEGATE)
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,14,0)
   heif_deinit();
 #endif
+#endif
 }
 
 /*
@@ -883,7 +1006,8 @@ static void WriteProfile(struct heif_context *context,Image *image,
     if (LocaleCompare(name,"XMP") == 0)
       for (i=0; i < (ssize_t) GetStringInfoLength(profile); i+=65533L)
       {
-        length=MagickMin(GetStringInfoLength(profile)-i,65533L);
+        length=(size_t) MagickMin((ssize_t) GetStringInfoLength(profile)-i,
+          65533L);
         error=heif_context_add_XMP_metadata(context,image_handle,
           (void*) (GetStringInfoDatum(profile)+i),(int) length);
         if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
@@ -961,14 +1085,14 @@ static MagickBooleanType WriteHEICImageYCbCr(Image *image,
   q_cr=heif_image_get_plane(heif_image,heif_channel_Cr,&p_cr);
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    const PixelPacket
+    const Quantum
       *p;
 
     ssize_t
       x;
 
     p=GetVirtualPixels(image,0,y,image->columns,1,exception);
-    if (p == (const PixelPacket *) NULL)
+    if (p == (const Quantum *) NULL)
       {
         status=MagickFalse;
         break;
@@ -976,20 +1100,20 @@ static MagickBooleanType WriteHEICImageYCbCr(Image *image,
     if ((y & 0x01) != 0)
       for (x=0; x < (ssize_t) image->columns; x++)
       {
-        q_y[y*p_y+x]=ScaleQuantumToChar(GetPixelRed(p));
-        p++;
+        q_y[y*p_y+x]=ScaleQuantumToChar(GetPixelRed(image,p));
+        p+=(ptrdiff_t) GetPixelChannels(image);
       }
     else
       for (x=0; x < (ssize_t) image->columns; x+=2)
       {
-        q_y[y*p_y+x]=ScaleQuantumToChar(GetPixelRed(p));
-        q_cb[y/2*p_cb+x/2]=ScaleQuantumToChar(GetPixelGreen(p));
-        q_cr[y/2*p_cr+x/2]=ScaleQuantumToChar(GetPixelBlue(p));
-        p++;
+        q_y[y*p_y+x]=ScaleQuantumToChar(GetPixelRed(image,p));
+        q_cb[y/2*p_cb+x/2]=ScaleQuantumToChar(GetPixelGreen(image,p));
+        q_cr[y/2*p_cr+x/2]=ScaleQuantumToChar(GetPixelBlue(image,p));
+        p+=(ptrdiff_t) GetPixelChannels(image);
         if ((x+1) < (ssize_t) image->columns)
           {
-            q_y[y*p_y+x+1]=ScaleQuantumToChar(GetPixelRed(p));
-            p++;
+            q_y[y*p_y+x+1]=ScaleQuantumToChar(GetPixelRed(image,p));
+            p+=(ptrdiff_t) GetPixelChannels(image);
           }
       }
     if (image->previous == (Image *) NULL)
@@ -1006,7 +1130,7 @@ static MagickBooleanType WriteHEICImageYCbCr(Image *image,
 static MagickBooleanType WriteHEICImageRGBA(Image *image,
   struct heif_image *heif_image,ExceptionInfo *exception)
 {
-  const PixelPacket
+  const Quantum
     *p;
 
   enum heif_channel
@@ -1033,7 +1157,7 @@ static MagickBooleanType WriteHEICImageRGBA(Image *image,
   */
   status=MagickTrue;
   channel=heif_channel_interleaved;
-  if (IsGrayImage(image,&image->exception) != MagickFalse)
+  if (GetPixelChannels(image) == 1)
     channel=heif_channel_Y;
   error=heif_image_add_plane(heif_image,channel,(int) image->columns,
     (int) image->rows,8);
@@ -1049,7 +1173,7 @@ static MagickBooleanType WriteHEICImageRGBA(Image *image,
       x;
 
     p=GetVirtualPixels(image,0,y,image->columns,1,exception);
-    if (p == (const PixelPacket *) NULL)
+    if (p == (const Quantum *) NULL)
       {
         status=MagickFalse;
         break;
@@ -1057,15 +1181,15 @@ static MagickBooleanType WriteHEICImageRGBA(Image *image,
     q=pixels+(y*stride);
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      *(q++)=ScaleQuantumToChar(GetPixelRed(p));
-      if (IsGrayImage(image,&image->exception) == MagickFalse)
+      *(q++)=ScaleQuantumToChar(GetPixelRed(image,p));
+      if (GetPixelChannels(image) > 1)
         {
-          *(q++)=ScaleQuantumToChar(GetPixelGreen(p));
-          *(q++)=ScaleQuantumToChar(GetPixelBlue(p));
-          if (image->matte != MagickFalse)
-            *(q++)=ScaleQuantumToChar(GetPixelAlpha(p));
+          *(q++)=ScaleQuantumToChar(GetPixelGreen(image,p));
+          *(q++)=ScaleQuantumToChar(GetPixelBlue(image,p));
+          if (image->alpha_trait != UndefinedPixelTrait)
+            *(q++)=ScaleQuantumToChar(GetPixelAlpha(image,p));
         }
-      p++;
+      p+=(ptrdiff_t) GetPixelChannels(image);
     }
     if (image->previous == (Image *) NULL)
       {
@@ -1081,7 +1205,7 @@ static MagickBooleanType WriteHEICImageRGBA(Image *image,
 static MagickBooleanType WriteHEICImageRRGGBBAA(Image *image,
   struct heif_image *heif_image,ExceptionInfo *exception)
 {
-  const PixelPacket
+  const Quantum
     *p;
 
   enum heif_channel
@@ -1109,7 +1233,7 @@ static MagickBooleanType WriteHEICImageRRGGBBAA(Image *image,
     Transform HEIF RGBA image with depth > 8.
   */
   depth=image->depth > 10 ? 12 : 10;
-  if (IsGrayImage(image,&image->exception) != MagickFalse)
+  if (GetPixelChannels(image) == 1)
     channel=heif_channel_Y;
   error=heif_image_add_plane(heif_image,channel,(int) image->columns,
     (int) image->rows,depth);
@@ -1128,7 +1252,7 @@ static MagickBooleanType WriteHEICImageRRGGBBAA(Image *image,
       x;
 
     p=GetVirtualPixels(image,0,y,image->columns,1,exception);
-    if (p == (const PixelPacket *) NULL)
+    if (p == (const Quantum *) NULL)
       {
         status=MagickFalse;
         break;
@@ -1136,25 +1260,25 @@ static MagickBooleanType WriteHEICImageRRGGBBAA(Image *image,
     q=pixels+(y*stride);
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      int pixel=ScaleQuantumToShort(GetPixelRed(p)) >> shift;
+      int pixel=ScaleQuantumToShort(GetPixelRed(image,p)) >> shift;
       *(q++)=(uint8_t) (pixel & 0xff);
       *(q++)=(uint8_t) (pixel >> 8);
-      if (IsGrayImage(image,&image->exception) == MagickFalse)
+      if (GetPixelChannels(image) > 1)
         {
-          pixel=ScaleQuantumToShort(GetPixelGreen(p)) >> shift;
+          pixel=ScaleQuantumToShort(GetPixelGreen(image,p)) >> shift;
           *(q++)=(uint8_t) (pixel & 0xff);
           *(q++)=(uint8_t) (pixel >> 8);
-          pixel=ScaleQuantumToShort(GetPixelBlue(p)) >> shift;
+          pixel=ScaleQuantumToShort(GetPixelBlue(image,p)) >> shift;
           *(q++)=(uint8_t) (pixel & 0xff);
           *(q++)=(uint8_t) (pixel >> 8);
-          if (image->matte != MagickFalse)
+          if (image->alpha_trait != UndefinedPixelTrait)
             {
-              pixel=ScaleQuantumToShort(GetPixelAlpha(p)) >> shift;
+              pixel=ScaleQuantumToShort(GetPixelAlpha(image,p)) >> shift;
               *(q++)=(uint8_t) (pixel & 0xff);
               *(q++)=(uint8_t) (pixel >> 8);
             }
         }
-      p++;
+      p+=(ptrdiff_t) GetPixelChannels(image);
     }
     if (image->previous == (Image *) NULL)
       {
@@ -1168,10 +1292,10 @@ static MagickBooleanType WriteHEICImageRRGGBBAA(Image *image,
 }
 
 static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
-  Image *image)
+  Image *image,ExceptionInfo *exception)
 {
   MagickBooleanType
-#if LIBHEIF_NUMERIC_VERSION > 0x01060200
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,7,0)
     encode_avif,
 #endif
     status;
@@ -1200,17 +1324,20 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
   assert(image->signature == MagickCoreSignature);
   if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
   scene=0;
   heif_context=heif_context_alloc();
-#if LIBHEIF_NUMERIC_VERSION > 0x01060200
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,7,0)
   encode_avif=(LocaleCompare(image_info->magick,"AVIF") == 0) ? MagickTrue :
     MagickFalse;
 #endif
   do
   {
+    const char
+      *option;
+
     const StringInfo
       *profile;
 
@@ -1229,7 +1356,7 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
     /*
       Get encoder for the specified format.
     */
-#if LIBHEIF_NUMERIC_VERSION > 0x01060200
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,7,0)
     if (encode_avif != MagickFalse)
       error=heif_context_get_encoder_for_format(heif_context,
         heif_compression_AV1,&heif_encoder);
@@ -1237,16 +1364,16 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
 #endif
       error=heif_context_get_encoder_for_format(heif_context,
         heif_compression_HEVC,&heif_encoder);
-    if (IsHEIFSuccess(image,&error,&image->exception) == MagickFalse)
+    if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
       break;
-    status=IsHEIFSuccess(image,&error,&image->exception);
+    status=IsHEIFSuccess(image,&error,exception);
     if (status == MagickFalse)
       break;
     chroma=lossless != MagickFalse ? heif_chroma_444 : heif_chroma_420;
-    if (image->matte != MagickFalse)
+    if ((image->alpha_trait & BlendPixelTrait) != 0)
       {
         if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-          status=TransformImageColorspace(image,sRGBColorspace);
+          status=TransformImageColorspace(image,sRGBColorspace,exception);
         colorspace=heif_colorspace_RGB;
         chroma=heif_chroma_interleaved_RGBA;
         if (image->depth > 8)
@@ -1259,7 +1386,7 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
           chroma=heif_chroma_interleaved_RGB;
           if (image->depth > 8)
             chroma=heif_chroma_interleaved_RRGGBB_LE;
-          if (IsGrayImage(image,&image->exception) != MagickFalse)
+          if (GetPixelChannels(image) == 1)
             {
               colorspace=heif_colorspace_monochrome;
               chroma=heif_chroma_monochrome;
@@ -1267,7 +1394,7 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
         }
       else
         if (image->colorspace != YCbCrColorspace)
-          status=TransformImageColorspace(image,YCbCrColorspace);
+          status=TransformImageColorspace(image,YCbCrColorspace,exception);
     if (status == MagickFalse)
       break;
     /*
@@ -1275,9 +1402,9 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
     */
     error=heif_image_create((int) image->columns,(int) image->rows,colorspace,
       chroma,&heif_image);
-    if (IsHEIFSuccess(image,&error,&image->exception) == MagickFalse)
+    if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
       break;
-    status=IsHEIFSuccess(image,&error,&image->exception);
+    status=IsHEIFSuccess(image,&error,exception);
     if (status == MagickFalse)
       break;
     profile=GetImageProfile(image,"icc");
@@ -1285,12 +1412,12 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
       (void) heif_image_set_raw_color_profile(heif_image,"prof",
         GetStringInfoDatum(profile),GetStringInfoLength(profile));
     if (colorspace == heif_colorspace_YCbCr)
-      status=WriteHEICImageYCbCr(image,heif_image,&image->exception);
+      status=WriteHEICImageYCbCr(image,heif_image,exception);
     else
       if (image->depth > 8)
-        status=WriteHEICImageRRGGBBAA(image,heif_image,&image->exception);
+        status=WriteHEICImageRRGGBBAA(image,heif_image,exception);
       else
-        status=WriteHEICImageRGBA(image,heif_image,&image->exception);
+        status=WriteHEICImageRGBA(image,heif_image,exception);
     if (status == MagickFalse)
       break;
     /*
@@ -1301,48 +1428,70 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
     else if (image_info->quality != UndefinedCompressionQuality)
       error=heif_encoder_set_lossy_quality(heif_encoder,(int)
         image_info->quality);
-    status=IsHEIFSuccess(image,&error,&image->exception);
+    status=IsHEIFSuccess(image,&error,exception);
     if (status == MagickFalse)
       break;
-#if LIBHEIF_NUMERIC_VERSION > 0x01060200
-    if (encode_avif != MagickFalse)
+    option=GetImageOption(image_info,"heic:speed");
+    if (option != (char *) NULL)
       {
-        const char
-          *option;
-
-        option=GetImageOption(image_info,"heic:speed");
-        if (option != (char *) NULL)
+        error=heif_encoder_set_parameter(heif_encoder,"speed",option);
+        status=IsHEIFSuccess(image,&error,exception);
+        if (status == MagickFalse)
+          break;
+      }
+    option=GetImageOption(image_info,"heic:chroma");
+    if (option != (char *) NULL)
+      {
+        error=heif_encoder_set_parameter(heif_encoder,"chroma",option);
+        status=IsHEIFSuccess(image,&error,exception);
+        if (status == MagickFalse)
+          break;
+      }
+    options=heif_encoding_options_alloc();
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,16,0)
+    option=GetImageOption(image_info,"heic:chroma-downsampling");
+    if (option != (char *) NULL)
+      {
+        if (LocaleCompare(option,"nearest-neighbor") == 0)
           {
-            error=heif_encoder_set_parameter(heif_encoder,"speed",option);
-            status=IsHEIFSuccess(image,&error,&image->exception);
-            if (status == MagickFalse)
-              break;
+            options->color_conversion_options.
+              only_use_preferred_chroma_algorithm=1;
+            options->color_conversion_options.
+              preferred_chroma_downsampling_algorithm=
+              heif_chroma_downsampling_nearest_neighbor;
           }
-        option=GetImageOption(image_info,"heic:chroma");
-        if (option != (char *) NULL)
+        else if (LocaleCompare(option,"average") == 0)
           {
-            error=heif_encoder_set_parameter(heif_encoder,"chroma",option);
-            status=IsHEIFSuccess(image,&error,&image->exception);
-            if (status == MagickFalse)
-              break;
+            options->color_conversion_options.
+              only_use_preferred_chroma_algorithm=1;
+            options->color_conversion_options.
+              preferred_chroma_downsampling_algorithm=
+              heif_chroma_downsampling_average;
+          }
+        else if (LocaleCompare(option,"sharp-yuv") == 0)
+          {
+            options->color_conversion_options.
+              only_use_preferred_chroma_algorithm=1;
+            options->color_conversion_options.
+              preferred_chroma_downsampling_algorithm=
+              heif_chroma_downsampling_sharp_yuv;
           }
       }
 #endif
-    options=heif_encoding_options_alloc();
-#if LIBHEIF_NUMERIC_VERSION >= 0x010e0000
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,14,0)
     if (image->orientation != UndefinedOrientation)
       options->image_orientation=(enum heif_orientation) image->orientation;
 #endif
     error=heif_context_encode_image(heif_context,heif_image,heif_encoder,
       options,(struct heif_image_handle **) NULL);
     heif_encoding_options_free(options);
-    if (IsHEIFSuccess(image,&error,&image->exception) == MagickFalse)
+    if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
       break;
-    status=IsHEIFSuccess(image,&error,&image->exception);
+    status=IsHEIFSuccess(image,&error,exception);
     if (status == MagickFalse)
       break;
     if (image->profiles != (void *) NULL)
-      WriteProfile(heif_context,image,&image->exception);
+      WriteProfile(heif_context,image,exception);
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
@@ -1364,14 +1513,15 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
       writer.writer_api_version=1;
       writer.write=heif_write_func;
       error=heif_context_write(heif_context,&writer,image);
-      status=IsHEIFSuccess(image,&error,&image->exception);
+      status=IsHEIFSuccess(image,&error,exception);
     }
   if (heif_encoder != (struct heif_encoder*) NULL)
     heif_encoder_release(heif_encoder);
   if (heif_image != (struct heif_image*) NULL)
     heif_image_release(heif_image);
   heif_context_free(heif_context);
-  (void) CloseBlob(image);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
   return(status);
 }
 #endif
