@@ -18,7 +18,7 @@
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -36,47 +36,33 @@
 %
 %
 */
-
-
+
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/constitute.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/module.h"
-#include "magick/pixel-accessor.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/resource_.h"
-#include "magick/string_.h"
-#include "magick/utility.h"
-#if defined(MAGICKCORE_XML_DELEGATE)
-#  if defined(MAGICKCORE_WINDOWS_SUPPORT)
-#    if !defined(__MINGW32__)
-#      include <win32config.h>
-#    endif
-#  endif
-#  include <libxml/parser.h>
-#  include <libxml/xmlmemory.h>
-#if defined(LIBXML_FTP_ENABLED)
-#  include <libxml/nanoftp.h>
-#endif
-#if defined(LIBXML_HTTP_ENABLED)
-#  include <libxml/nanohttp.h>
-#endif
-#endif
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/constitute.h"
+#include "MagickCore/delegate.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/utility.h"
 #if defined(MAGICKCORE_WINDOWS_SUPPORT)
 #  include <urlmon.h>
-#  pragma comment(lib, "urlmon.lib")
+#  if !defined(__MINGW32__)
+#    pragma comment(lib, "urlmon.lib")
+#  endif
 #endif
 
 /*
@@ -106,39 +92,10 @@
 %
 */
 
-#if defined(__cplusplus) || defined(c_plusplus)
-extern "C" {
-#endif
-
-#if defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_FTP_ENABLED)
-static void GetFTPData(void *userdata,const char *data,int size)
-{
-  FILE
-    *file;
-
-  size_t
-    length;
-
-  file=(FILE *) userdata;
-  if (file == (FILE *) NULL)
-    return;
-  if (size <= 0)
-    return;
-  length=fwrite(data,size,1,file);
-  (void) length;
-}
-#endif
-
-#if defined(__cplusplus) || defined(c_plusplus)
-}
-#endif
-
 static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-#define MaxBufferExtent  8192
-
   char
-    filename[MaxTextExtent];
+    filename[MagickPathExtent];
 
   FILE
     *file;
@@ -155,7 +112,7 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     unique_file;
 
   images=(Image *) NULL;
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   read_info=CloneImageInfo(image_info);
   SetImageInfoBlob(read_info,(void *) NULL,0);
 #if !defined(MAGICKCORE_WINDOWS_SUPPORT)
@@ -179,7 +136,7 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (images != (Image *) NULL)
             for (next=images; next != (Image *) NULL; next=next->next)
               (void) CopyMagickString(next->filename,image->filename,
-                MaxTextExtent);
+                MagickPathExtent);
         }
       read_info=DestroyImageInfo(read_info);
       image=DestroyImage(image);
@@ -189,7 +146,7 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (LocaleCompare(read_info->magick,"file") == 0)
     {
       (void) CopyMagickString(read_info->filename,image_info->filename+2,
-        MaxTextExtent);
+        MagickPathExtent);
       *read_info->magick='\0';
       images=ReadImage(read_info,exception);
       read_info=DestroyImageInfo(read_info);
@@ -208,10 +165,11 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImage(image);
       return((Image *) NULL);
     }
-  (void) CopyMagickString(filename,image_info->magick,MaxTextExtent);
-  (void) ConcatenateMagickString(filename,":",MaxTextExtent);
+  (void) CopyMagickString(filename,image_info->magick,MagickPathExtent);
+  (void) ConcatenateMagickString(filename,":",MagickPathExtent);
   LocaleLower(filename);
-  (void) ConcatenateMagickString(filename,image_info->filename,MaxTextExtent);
+  (void) ConcatenateMagickString(filename,image_info->filename,
+    MagickPathExtent);
 #if defined(MAGICKCORE_WINDOWS_SUPPORT)
   (void) fclose(file);
   if (URLDownloadToFile(NULL,filename,read_info->filename,0,NULL) != S_OK)
@@ -224,53 +182,6 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
 #else
-#if defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_FTP_ENABLED)
-  if (LocaleCompare(read_info->magick,"ftp") == 0)
-    {
-      void
-        *context;
-
-      xmlNanoFTPInit();
-      context=xmlNanoFTPNewCtxt(filename);
-      if (context != (void *) NULL)
-        {
-          if (xmlNanoFTPConnect(context) >= 0)
-            (void) xmlNanoFTPGet(context,GetFTPData,(void *) file,
-              (char *) NULL);
-          (void) xmlNanoFTPClose(context);
-        }
-    }
-#endif
-#if defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_HTTP_ENABLED)
-  if (LocaleCompare(read_info->magick,"http") == 0)
-    {
-      char
-        buffer[MaxBufferExtent],
-        *type;
-
-      int
-        bytes;
-
-      void
-        *context;
-
-      type=(char *) NULL;
-      context=xmlNanoHTTPMethod(filename,(const char *) NULL,
-        (const char *) NULL,&type,(const char *) NULL,0);
-      if (context != (void *) NULL)
-        {
-          ssize_t
-            count;
-
-          while ((bytes=xmlNanoHTTPRead(context,buffer,MaxBufferExtent)) > 0)
-            count=(ssize_t) fwrite(buffer,bytes,1,file);
-          (void) count;
-          xmlNanoHTTPClose(context);
-          xmlFree(type);
-          xmlNanoHTTPCleanup();
-        }
-    }
-#endif
   (void) fclose(file);
 #endif
   *read_info->magick='\0';
@@ -278,8 +189,9 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) RelinquishUniqueFileResource(read_info->filename);
   if (images != (Image *) NULL)
     for (next=images; next != (Image *) NULL; next=next->next)
-      (void) CopyMagickString(next->filename,image->filename,MaxTextExtent);
+      (void) CopyMagickString(next->filename,image->filename,MagickPathExtent);
   read_info=DestroyImageInfo(read_info);
+  image=DestroyImage(image);
   if (images != (Image *) NULL)
     GetPathComponent(image_info->filename,TailPath,images->filename);
   else
@@ -288,7 +200,6 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         "NoDataReturned","`%s'",filename);
       return((Image *) NULL);
     }
-  image=DestroyImage(image);
   return(GetFirstImageInList(images));
 }
 
@@ -320,40 +231,29 @@ ModuleExport size_t RegisterURLImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("HTTP");
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) || \
-    (defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_HTTP_ENABLED))
+  entry=AcquireMagickInfo("URL","HTTP","Uniform Resource Locator (http://)");
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
 #endif
-  entry->description=ConstantString("Uniform Resource Locator (http://)");
-  entry->magick_module=ConstantString("URL");
   entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("HTTPS");
+  entry=AcquireMagickInfo("URL","HTTPS","Uniform Resource Locator (https://)");
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
-  entry->description=ConstantString("Uniform Resource Locator (https://)");
-  entry->magick_module=ConstantString("URL");
   entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("FTP");
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) || \
-    (defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_FTP_ENABLED))
+  entry=AcquireMagickInfo("URL","FTP","Uniform Resource Locator (ftp://)");
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
 #endif
-  entry->description=ConstantString("Uniform Resource Locator (ftp://)");
-  entry->magick_module=ConstantString("URL");
   entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("FILE");
+  entry=AcquireMagickInfo("URL","FILE","Uniform Resource Locator (file://)");
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
-  entry->description=ConstantString("Uniform Resource Locator (file://)");
-  entry->magick_module=ConstantString("URL");
   entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -376,7 +276,6 @@ ModuleExport size_t RegisterURLImage(void)
 ModuleExport void UnregisterURLImage(void)
 {
   (void) UnregisterMagickInfo("HTTP");
-  (void) UnregisterMagickInfo("HTTPS");
   (void) UnregisterMagickInfo("FTP");
   (void) UnregisterMagickInfo("FILE");
 }
