@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -39,35 +39,35 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/colorspace.h"
-#include "magick/constitute.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/geometry.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/pixel-accessor.h"
-#include "magick/quantum-private.h"
-#include "magick/resize.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/colorspace.h"
+#include "MagickCore/constitute.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/geometry.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/resize.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/utility.h"
 
 /*
   Forward declarations.
 */
 static MagickBooleanType
-  WriteYUVImage(const ImageInfo *,Image *);
+  WriteYUVImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -109,13 +109,10 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  const PixelPacket
-    *chroma_pixels;
-
   ssize_t
     x;
 
-  PixelPacket
+  Quantum
     *q;
 
   unsigned char
@@ -144,15 +141,12 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
-  status=SetImageExtent(image,image->columns,image->rows);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
+    return(DestroyImageList(image));
   quantum=(size_t) (image->depth <= 8 ? 1 : 2);
   interlace=image_info->interlace;
   horizontal_factor=2;
@@ -201,28 +195,32 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Allocate memory for a scanline.
   */
   if (interlace == NoInterlace)
-    scanline=(unsigned char *) AcquireQuantumMemory((size_t) 2UL*
-      image->columns+2UL,quantum*sizeof(*scanline));
+    scanline=(unsigned char *) AcquireQuantumMemory((size_t) (2UL*
+      image->columns+2UL),(size_t) quantum*sizeof(*scanline));
   else
     scanline=(unsigned char *) AcquireQuantumMemory(image->columns,
-      quantum*sizeof(*scanline));
+      (size_t) quantum*sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   status=MagickTrue;
   do
   {
-    chroma_image=CloneImage(image,(image->columns+horizontal_factor-1)/
-      horizontal_factor,(image->rows+vertical_factor-1)/vertical_factor,
-      MagickTrue,exception);
+    chroma_image=CloneImage(image,(size_t) ((ssize_t) image->columns+
+      (ssize_t) horizontal_factor-1)/(size_t) horizontal_factor,(size_t)
+      (image->rows+(size_t) vertical_factor-1)/
+      (size_t) vertical_factor,MagickTrue,exception);
     if (chroma_image == (Image *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      {
+        scanline=(unsigned char *) RelinquishMagickMemory(scanline); 
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      }
     /*
       Convert raster image to pixel packets.
     */
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    status=SetImageExtent(image,image->columns,image->rows);
+    status=SetImageExtent(image,image->columns,image->rows,exception);
     if (status == MagickFalse)
       break;
     if (interlace == PartitionInterlace)
@@ -231,13 +229,14 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
         if (status == MagickFalse)
           {
+            scanline=(unsigned char *) RelinquishMagickMemory(scanline); 
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
       }
     for (y=0; y < (ssize_t) image->rows; y++)
     {
-      PixelPacket
+      Quantum
         *chroma_pixels;
 
       if (interlace == NoInterlace)
@@ -256,52 +255,53 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
             }
           p=scanline;
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-          if (q == (PixelPacket *) NULL)
+          if (q == (Quantum *) NULL)
             break;
           chroma_pixels=QueueAuthenticPixels(chroma_image,0,y,
             chroma_image->columns,1,exception);
-          if (chroma_pixels == (PixelPacket *) NULL)
+          if (chroma_pixels == (Quantum *) NULL)
             break;
           for (x=0; x < (ssize_t) image->columns; x+=2)
           {
-            SetPixelRed(chroma_pixels,0);
+            SetPixelRed(chroma_image,0,chroma_pixels);
             if (quantum == 1)
-              SetPixelGreen(chroma_pixels,ScaleCharToQuantum(*p++));
+              SetPixelGreen(chroma_image,ScaleCharToQuantum(*p++),
+                chroma_pixels);
             else
               {
-                SetPixelGreen(chroma_pixels,ScaleShortToQuantum(((*p) << 8) |
-                  *(p+1)));
-                p+=2;
+                SetPixelGreen(chroma_image,ScaleShortToQuantum(((*p) << 8) |
+                  *(p+1)),chroma_pixels);
+                p+=(ptrdiff_t) 2;
               }
             if (quantum == 1)
-              SetPixelRed(q,ScaleCharToQuantum(*p++));
+              SetPixelRed(image,ScaleCharToQuantum(*p++),q);
             else
               {
-                SetPixelRed(q,ScaleShortToQuantum(((*p) << 8) | *(p+1)));
-                p+=2;
+                SetPixelRed(image,ScaleShortToQuantum(((*p) << 8) | *(p+1)),q);
+                p+=(ptrdiff_t) 2;
               }
-            SetPixelGreen(q,0);
-            SetPixelBlue(q,0);
-            q++;
-            SetPixelGreen(q,0);
-            SetPixelBlue(q,0);
+            SetPixelGreen(image,0,q);
+            SetPixelBlue(image,0,q);
+            q+=(ptrdiff_t) GetPixelChannels(image);
+            SetPixelGreen(image,0,q);
+            SetPixelBlue(image,0,q);
             if (quantum == 1)
-              SetPixelBlue(chroma_pixels,ScaleCharToQuantum(*p++));
+              SetPixelBlue(chroma_image,ScaleCharToQuantum(*p++),chroma_pixels);
             else
               {
-                SetPixelBlue(chroma_pixels,ScaleShortToQuantum(((*p) << 8) |
-                  *(p+1)));
-                p+=2;
+                SetPixelBlue(chroma_image,ScaleShortToQuantum(((*p) << 8) |
+                  *(p+1)),chroma_pixels);
+                p+=(ptrdiff_t) 2;
               }
             if (quantum == 1)
-              SetPixelRed(q,ScaleCharToQuantum(*p++));
+              SetPixelRed(image,ScaleCharToQuantum(*p++),q);
             else
               {
-                SetPixelRed(q,ScaleShortToQuantum(((*p) << 8) | *(p+1)));
-                p+=2;
+                SetPixelRed(image,ScaleShortToQuantum(((*p) << 8) | *(p+1)),q);
+                p+=(ptrdiff_t) 2;
               }
-            chroma_pixels++;
-            q++;
+            chroma_pixels+=(ptrdiff_t) GetPixelChannels(chroma_image);
+            q+=(ptrdiff_t) GetPixelChannels(image);
           }
         }
       else
@@ -320,20 +320,20 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
             }
           p=scanline;
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-          if (q == (PixelPacket *) NULL)
+          if (q == (Quantum *) NULL)
             break;
           for (x=0; x < (ssize_t) image->columns; x++)
           {
             if (quantum == 1)
-              SetPixelRed(q,ScaleCharToQuantum(*p++));
+              SetPixelRed(image,ScaleCharToQuantum(*p++),q);
             else
               {
-                SetPixelRed(q,ScaleShortToQuantum(((*p) << 8) | *(p+1)));
-                p+=2;
+                SetPixelRed(image,ScaleShortToQuantum(((*p) << 8) | *(p+1)),q);
+                p+=(ptrdiff_t) 2;
               }
-            SetPixelGreen(q,0);
-            SetPixelBlue(q,0);
-            q++;
+            SetPixelGreen(image,0,q);
+            SetPixelBlue(image,0,q);
+            q+=(ptrdiff_t) GetPixelChannels(image);
           }
         }
       if (SyncAuthenticPixels(image,exception) == MagickFalse)
@@ -357,6 +357,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
         if (status == MagickFalse)
           {
+            scanline=(unsigned char *) RelinquishMagickMemory(scanline); 
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
@@ -377,20 +378,21 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           p=scanline;
           q=QueueAuthenticPixels(chroma_image,0,y,chroma_image->columns,1,
             exception);
-          if (q == (PixelPacket *) NULL)
+          if (q == (Quantum *) NULL)
             break;
           for (x=0; x < (ssize_t) chroma_image->columns; x++)
           {
-            SetPixelRed(q,0);
+            SetPixelRed(chroma_image,0,q);
             if (quantum == 1)
-              SetPixelGreen(q,ScaleCharToQuantum(*p++));
+              SetPixelGreen(chroma_image,ScaleCharToQuantum(*p++),q);
             else
               {
-                SetPixelGreen(q,ScaleShortToQuantum(((*p) << 8) | *(p+1)));
-                p+=2;
+                SetPixelGreen(chroma_image,ScaleShortToQuantum(((*p) << 8) |
+                  *(p+1)),q);
+                p+=(ptrdiff_t) 2;
               }
-            SetPixelBlue(q,0);
-            q++;
+            SetPixelBlue(chroma_image,0,q);
+            q+=(ptrdiff_t) GetPixelChannels(chroma_image);
           }
           if (SyncAuthenticPixels(chroma_image,exception) == MagickFalse)
             break;
@@ -403,6 +405,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
           if (status == MagickFalse)
             {
+              scanline=(unsigned char *) RelinquishMagickMemory(scanline); 
               image=DestroyImageList(image);
               return((Image *) NULL);
             }
@@ -421,18 +424,19 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         p=scanline;
         q=GetAuthenticPixels(chroma_image,0,y,chroma_image->columns,1,
           exception);
-        if (q == (PixelPacket *) NULL)
+        if (q == (Quantum *) NULL)
           break;
         for (x=0; x < (ssize_t) chroma_image->columns; x++)
         {
           if (quantum == 1)
-            SetPixelBlue(q,ScaleCharToQuantum(*p++));
+            SetPixelBlue(chroma_image,ScaleCharToQuantum(*p++),q);
           else
             {
-              SetPixelBlue(q,ScaleShortToQuantum(((*p) << 8) | *(p+1)));
-              p+=2;
+              SetPixelBlue(chroma_image,ScaleShortToQuantum(((*p) << 8) |
+                *(p+1)),q);
+              p+=(ptrdiff_t) 2;
             }
-          q++;
+          q+=(ptrdiff_t) GetPixelChannels(chroma_image);
         }
         if (SyncAuthenticPixels(chroma_image,exception) == MagickFalse)
           break;
@@ -442,34 +446,40 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Scale image.
     */
     resize_image=ResizeImage(chroma_image,image->columns,image->rows,
-      TriangleFilter,1.0,exception);
+      TriangleFilter,exception);
     chroma_image=DestroyImage(chroma_image);
     if (resize_image == (Image *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      {
+        scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      }
     for (y=0; y < (ssize_t) image->rows; y++)
     {
+      const Quantum
+        *chroma_pixels;
+
       q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
       chroma_pixels=GetVirtualPixels(resize_image,0,y,resize_image->columns,1,
-        &resize_image->exception);
-      if ((q == (PixelPacket *) NULL) ||
-          (chroma_pixels == (const PixelPacket *) NULL))
+        exception);
+      if ((q == (Quantum *) NULL) ||
+          (chroma_pixels == (const Quantum *) NULL))
         break;
       for (x=0; x < (ssize_t) image->columns; x++)
       {
-        SetPixelGreen(q,GetPixelGreen(chroma_pixels));
-        SetPixelBlue(q,GetPixelBlue(chroma_pixels));
-        chroma_pixels++;
-        q++;
+        SetPixelGreen(image,GetPixelGreen(resize_image,chroma_pixels),q);
+        SetPixelBlue(image,GetPixelBlue(resize_image,chroma_pixels),q);
+        chroma_pixels+=(ptrdiff_t) GetPixelChannels(resize_image);
+        q+=(ptrdiff_t) GetPixelChannels(image);
       }
       if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
     }
     resize_image=DestroyImage(resize_image);
-    if (SetImageColorspace(image,YCbCrColorspace) == MagickFalse)
+    if (SetImageColorspace(image,YCbCrColorspace,exception) == MagickFalse)
       break;
     if (interlace == PartitionInterlace)
       (void) CopyMagickString(image->filename,image_info->filename,
-        MaxTextExtent);
+        MagickPathExtent);
     if (EOFBlob(image) != MagickFalse)
       {
         ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
@@ -491,7 +501,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Allocate next image structure.
         */
-        AcquireNextImage(image_info,image);
+        AcquireNextImage(image_info,image,exception);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             status=MagickFalse;
@@ -505,7 +515,6 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
   } while (count != 0);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
-  InheritException(exception,&image->exception);
   if (CloseBlob(image) == MagickFalse)
     status=MagickFalse;
   if (status == MagickFalse)
@@ -541,13 +550,11 @@ ModuleExport size_t RegisterYUVImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("YUV");
+  entry=AcquireMagickInfo("YUV","YUV","CCIR 601 4:1:1 or 4:2:2");
   entry->decoder=(DecodeImageHandler *) ReadYUVImage;
   entry->encoder=(EncodeImageHandler *) WriteYUVImage;
-  entry->adjoin=MagickFalse;
-  entry->raw=MagickTrue;
-  entry->description=ConstantString("CCIR 601 4:1:1 or 4:2:2");
-  entry->magick_module=ConstantString("YUV");
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags|=CoderRawSupportFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -593,7 +600,8 @@ ModuleExport void UnregisterYUVImage(void)
 %
 %  The format of the WriteYUVImage method is:
 %
-%      MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteYUVImage(const ImageInfo *image_info,
+%        Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -601,8 +609,11 @@ ModuleExport void UnregisterYUVImage(void)
 %
 %    o image:  The image.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
-static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
   Image
     *chroma_image,
@@ -617,7 +628,7 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
   MagickOffsetType
     scene;
 
-  const PixelPacket
+  const Quantum
     *p,
     *s;
 
@@ -675,14 +686,14 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
       /*
         Open output image file.
       */
-      status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+      status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
       if (status == MagickFalse)
         return(status);
     }
   else
     {
       AppendImageFormat("Y",image->filename);
-      status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+      status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
       if (status == MagickFalse)
         return(status);
     }
@@ -694,21 +705,27 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
       Sample image to an even width and height, if necessary.
     */
     image->depth=(size_t) (quantum == 1 ? 8 : 16);
-    width=image->columns+(image->columns & (horizontal_factor-1));
-    height=image->rows+(image->rows & (vertical_factor-1));
-    yuv_image=ResizeImage(image,width,height,TriangleFilter,1.0,
-      &image->exception);
+    width=image->columns+(image->columns & (unsigned int) (horizontal_factor
+      -1));
+    height=image->rows+(image->rows & (unsigned int) (vertical_factor-1));
+    yuv_image=ResizeImage(image,width,height,TriangleFilter,exception);
     if (yuv_image == (Image *) NULL)
-      ThrowWriterException(ResourceLimitError,image->exception.reason);
-    (void) TransformImageColorspace(yuv_image,YCbCrColorspace);
+      {
+        (void) CloseBlob(image);
+        return(MagickFalse);
+      }
+    (void) TransformImageColorspace(yuv_image,YCbCrColorspace,exception);
     /*
       Downsample image.
     */
-    chroma_image=ResizeImage(image,width/horizontal_factor,
-      height/vertical_factor,TriangleFilter,1.0,&image->exception);
+    chroma_image=ResizeImage(image,(size_t) ((ssize_t) width/horizontal_factor),
+      (size_t) ((ssize_t) height/vertical_factor),TriangleFilter,exception);
     if (chroma_image == (Image *) NULL)
-      ThrowWriterException(ResourceLimitError,image->exception.reason);
-    (void) TransformImageColorspace(chroma_image,YCbCrColorspace);
+      {
+        (void) CloseBlob(image);
+        return(MagickFalse);
+      }
+    (void) TransformImageColorspace(chroma_image,YCbCrColorspace,exception);
     if (interlace == NoInterlace)
       {
         /*
@@ -716,37 +733,40 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
         */
         for (y=0; y < (ssize_t) yuv_image->rows; y++)
         {
-          p=GetVirtualPixels(yuv_image,0,y,yuv_image->columns,1,
-            &yuv_image->exception);
-          if (p == (const PixelPacket *) NULL)
+          p=GetVirtualPixels(yuv_image,0,y,yuv_image->columns,1,exception);
+          if (p == (const Quantum *) NULL)
             break;
           s=GetVirtualPixels(chroma_image,0,y,chroma_image->columns,1,
-            &chroma_image->exception);
-          if (s == (const PixelPacket *) NULL)
+            exception);
+          if (s == (const Quantum *) NULL)
             break;
           for (x=0; x < (ssize_t) yuv_image->columns; x+=2)
           {
             if (quantum == 1)
               {
                 (void) WriteBlobByte(image,ScaleQuantumToChar(
-                  GetPixelGreen(s)));
-                (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelRed(p)));
-                p++;
-                (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelBlue(s)));
-                (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelRed(p)));
+                  GetPixelGreen(yuv_image,s)));
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetPixelRed(yuv_image,p)));
+                p+=(ptrdiff_t) GetPixelChannels(yuv_image);
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetPixelBlue(yuv_image,s)));
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetPixelRed(yuv_image,p)));
               }
             else
               {
                 (void) WriteBlobByte(image,ScaleQuantumToChar(
-                  GetPixelGreen(s)));
+                  GetPixelGreen(yuv_image,s)));
                 (void) WriteBlobShort(image,ScaleQuantumToShort(
-                  GetPixelRed(p)));
-                p++;
-                (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelBlue(s)));
+                  GetPixelRed(yuv_image,p)));
+                p+=(ptrdiff_t) GetPixelChannels(yuv_image);
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetPixelBlue(yuv_image,s)));
                 (void) WriteBlobShort(image,ScaleQuantumToShort(
-                  GetPixelRed(p)));
+                  GetPixelRed(yuv_image,p)));
               }
-            p++;
+            p+=(ptrdiff_t) GetPixelChannels(yuv_image);
             s++;
           }
           if (image->previous == (Image *) NULL)
@@ -766,17 +786,18 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
         */
         for (y=0; y < (ssize_t) yuv_image->rows; y++)
         {
-          p=GetVirtualPixels(yuv_image,0,y,yuv_image->columns,1,
-            &yuv_image->exception);
-          if (p == (const PixelPacket *) NULL)
+          p=GetVirtualPixels(yuv_image,0,y,yuv_image->columns,1,exception);
+          if (p == (const Quantum *) NULL)
             break;
           for (x=0; x < (ssize_t) yuv_image->columns; x++)
           {
             if (quantum == 1)
-              (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelRed(p)));
+              (void) WriteBlobByte(image,ScaleQuantumToChar(
+                GetPixelRed(yuv_image,p)));
             else
-              (void) WriteBlobShort(image,ScaleQuantumToShort(GetPixelRed(p)));
-            p++;
+              (void) WriteBlobShort(image,ScaleQuantumToShort(
+                GetPixelRed(yuv_image,p)));
+            p+=(ptrdiff_t) GetPixelChannels(yuv_image);
           }
           if (image->previous == (Image *) NULL)
             {
@@ -801,25 +822,25 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             if (CloseBlob(image) == MagickFalse)
               break;
             AppendImageFormat("U",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
-              &image->exception);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
             if (status == MagickFalse)
               return(status);
           }
         for (y=0; y < (ssize_t) chroma_image->rows; y++)
         {
           p=GetVirtualPixels(chroma_image,0,y,chroma_image->columns,1,
-            &chroma_image->exception);
-          if (p == (const PixelPacket *) NULL)
+            exception);
+          if (p == (const Quantum *) NULL)
             break;
           for (x=0; x < (ssize_t) chroma_image->columns; x++)
           {
             if (quantum == 1)
-              (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelGreen(p)));
+              (void) WriteBlobByte(image,ScaleQuantumToChar(
+                GetPixelGreen(chroma_image,p)));
             else
               (void) WriteBlobShort(image,ScaleQuantumToShort(
-                GetPixelGreen(p)));
-            p++;
+                GetPixelGreen(chroma_image,p)));
+            p+=(ptrdiff_t) GetPixelChannels(chroma_image);
           }
         }
         if (image->previous == (Image *) NULL)
@@ -836,24 +857,25 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             if (CloseBlob(image) == MagickFalse)
               break;
             AppendImageFormat("V",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
-              &image->exception);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
             if (status == MagickFalse)
               return(status);
           }
         for (y=0; y < (ssize_t) chroma_image->rows; y++)
         {
           p=GetVirtualPixels(chroma_image,0,y,chroma_image->columns,1,
-            &chroma_image->exception);
-          if (p == (const PixelPacket *) NULL)
+            exception);
+          if (p == (const Quantum *) NULL)
             break;
           for (x=0; x < (ssize_t) chroma_image->columns; x++)
           {
             if (quantum == 1)
-              (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelBlue(p)));
+              (void) WriteBlobByte(image,ScaleQuantumToChar(
+                GetPixelBlue(chroma_image,p)));
             else
-              (void) WriteBlobShort(image,ScaleQuantumToShort(GetPixelBlue(p)));
-            p++;
+              (void) WriteBlobShort(image,ScaleQuantumToShort(
+                GetPixelBlue(chroma_image,p)));
+            p+=(ptrdiff_t) GetPixelChannels(chroma_image);
           }
         }
         if (image->previous == (Image *) NULL)
@@ -866,7 +888,7 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
     chroma_image=DestroyImage(chroma_image);
     if (interlace == PartitionInterlace)
       (void) CopyMagickString(image->filename,image_info->filename,
-        MaxTextExtent);
+        MagickPathExtent);
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
