@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.  You may
@@ -33,7 +33,6 @@ static const char
   *composite_options[] =
   {
     "",
-    "-compose Add",
     "-compose Atop",
     "-compose Blend",
     "-compose Bumpmap",
@@ -71,6 +70,8 @@ static const char
     "-compose Luminize",
     "-compose Minus",
     "-compose Modulate",
+    "-compose ModulusAdd",
+    "-compose ModulusSubtract",
     "-compose Multiply",
     "-compose None",
     "-compose Out",
@@ -87,7 +88,6 @@ static const char
     "-compose SrcOut",
     "-compose SrcOver",
     "-compose Src",
-    "-compose Subtract",
     "-compose Threshold",
     "-compose Xor",
     "-geometry +35+65 -label Magick",
@@ -99,13 +99,15 @@ static const char
   {
     "",
     "-affine 1,0,0.785,1,0,0 -transform",
+    "-alpha set",
     "-black-threshold 20%",
     "-blur 0x0.5",
     "-blur 0x1.0",
     "-blur 0x2.0",
-    "-border 6x6",
+    "-bordercolor red -border 6x6",
     "-canny 0x1+10%+80%",
     "-channel red",
+    "-colorspace CMYK -channel Cyan",
     "-charcoal 0x1",
     "-chop 80x60+10+20",
     "-chop 8x6+20+30",
@@ -124,16 +126,16 @@ static const char
     "-colorspace YCbCr",
     "-colorspace YIQ",
     "-colorspace YUV",
-    "+contrast +contrast +contrast",
-    "-contrast -contrast -contrast",
-    "+contrast",
-    "-contrast",
     "-convolve 1,1,1,1,4,1,1,1,1",
     "-crop 17x9+10+10",
     "-crop 60x70+10+10",
     "-cycle 200",
     "-density 75x75 -resample 50x50",
+    "-depth 7",
+    "-depth 16",
+    "-depth 32",
     "-despeckle",
+    "-draw \"affine 1,0,0.785,1,0,0\"",
     "-draw \"rectangle 20,10 80,50\"",
     "-edge 0x1",
     "-emboss 0x1",
@@ -167,24 +169,23 @@ static const char
     "-fuzz 5% -trim",
     "-fx \"(1.0/(1.0+exp(10.0*(0.5-u)))-0.006693)*1.0092503\"",
     "-gamma 1.6",
-    "-gaussian 0x0.5",
-    "-gaussian 0x1.0",
-    "-gaussian 0x2.0",
+    "-gaussian-blur 0x0.5",
+    "-gaussian-blur 0x1.0",
+    "-gaussian-blur 0x2.0",
     "-implode 0.5",
     "-implode -1",
     "-label Magick",
     "-lat 10x10-5%",
+    "-level 100%",
     "-level 10%,1.2,90%",
+    "+level 100% +level 100% +level 100%",
+    "-level 100% -level 100% -level 100%",
+    "+level 100%",
     "-magnify",
-    "-map netscape:",
-    "-median 1",
-    "-median 2",
     "-modulate 110/100/95",
     "-monochrome",
     "-motion-blur 0x3+30",
     "-negate",
-    "-noise 1",
-    "-noise 2",
     "+noise Gaussian",
     "+noise Impulse",
     "+noise Laplacian",
@@ -194,10 +195,14 @@ static const char
     "+noise Uniform",
     "-noop",
     "-normalize",
+    "-ordered-dither 2x2",
+    "-ordered-dither 3x3",
+    "-ordered-dither 4x4",
     "-paint 0x1",
     "-raise 10x10",
     "-random-threshold 10%",
     "-random-threshold 20x80",
+    "-remap netscape:",
     "-resize 100%",
     "-resize 10%",
     "-resize 150%",
@@ -236,6 +241,10 @@ static const char
     "-solarize 50%",
     "-spread 1",
     "-spread 3",
+    "-statistic Median 1",
+    "-statistic Median 2",
+    "-statistic NonPeak 1",
+    "-statistic NonPeak 2",
     "-swirl 90",
     "-threshold 35%",
     "-trim",
@@ -317,7 +326,6 @@ static const struct ReferenceFormats
     { "GIF", UndefinedCompression, 0.0 },
     { "GIF87", UndefinedCompression, 0.0 },
     { "GRAY", UndefinedCompression, 0.004 },
-    { "HRZ", UndefinedCompression, 0.0 },
     { "HTM", UndefinedCompression, 0.0 },
     { "HTML", UndefinedCompression, 0.0 },
     { "ICB", UndefinedCompression, 0.0 },
@@ -356,9 +364,7 @@ static const struct ReferenceFormats
     { "PFM", UndefinedCompression, 0.004 },
     { "PGM", UndefinedCompression, 0.0 },
     { "PGX", UndefinedCompression, 0.0 },
-#if !defined(MAGICKCORE_HDRI_SUPPORT)
     { "PICT", UndefinedCompression, 0.004 },
-#endif
     { "PIX", UndefinedCompression, 0.0 },
     { "PJPEG", UndefinedCompression, 0.004 },
     { "PLASMA", UndefinedCompression, 0.0 },
@@ -396,6 +402,7 @@ static const struct ReferenceFormats
     { "TIFF64", UndefinedCompression, 0.0 },
     { "TILE", UndefinedCompression, 0.0 },
     { "TIM", UndefinedCompression, 0.0 },
+    { "TIM2", UndefinedCompression, 0.0 },
     { "TTC", UndefinedCompression, 0.0 },
     { "TTF", UndefinedCompression, 0.0 },
     { "TXT", UndefinedCompression, 0.0 },
@@ -479,8 +486,8 @@ static const struct ReferenceStorage
     { CharPixel, sizeof(unsigned char) },
     { DoublePixel, sizeof(double) },
     { FloatPixel, sizeof(float) },
-    { IntegerPixel, sizeof(unsigned int) },
-    { LongPixel, sizeof(size_t) },
+    { LongPixel, sizeof(unsigned int) },
+    { LongLongPixel, sizeof(MagickSizeType) },
     { ShortPixel, sizeof(unsigned short) },
     { UndefinedPixel, 0 }
   };
@@ -498,15 +505,15 @@ static const struct ReferenceTypes
   reference_types[] =
   {
     { TrueColorType, 8 },
-    { TrueColorMatteType, 8 },
+    { TrueColorAlphaType, 8 },
     { GrayscaleType, 8 },
-    { GrayscaleMatteType, 8 },
+    { GrayscaleAlphaType, 8 },
     { PaletteType, 8 },
-    { PaletteMatteType, 8 },
-    { PaletteBilevelMatteType, 8 },
+    { PaletteAlphaType, 8 },
+    { PaletteBilevelAlphaType, 8 },
     { BilevelType, 1 },
     { ColorSeparationType, 8 },
-    { ColorSeparationMatteType, 8 },
+    { ColorSeparationAlphaType, 8 },
     { TrueColorType, 10 },
     { TrueColorType, 12 },
     { TrueColorType, 16 },
